//
//  Analyse Frquentielle
//
// G. Dietrich 2017
require("jsConfig")
//------------------------------------
// FFT
//------------------------------------
function fhGetPow2(n)
{
    if (n > 8)
    {
        var nn = 2;
        while (nn < n)
        {
            nn = nn*2;
        }
        if (nn > n)
            nn = nn/2;
        else nn = n;
        return nn;
    }
    return 0;
}

function fhMean(raw_data)
{
    if (raw_data)
    {
        var size = raw_data.size();
        var i;
        var mean = 0.0;
        for(i = 0; i < size; i++)
        {
            mean = mean + raw_data.get(i);
        }
        mean = mean/size;
        return mean;
    }
    return 0;
}

function fhCenter(raw_data)
{
    if (raw_data)
    {
        var size = raw_data.size();
        var c_data = new Float64Vector(size);
        var mean = fhMean(raw_data);
        var i;
        for(i = 0; i < size; i++)
        {
            var d = raw_data.get(i) - mean;
            c_data.set(i, d);
        }
        return c_data;
    }
}

function fhCut(raw_data)
{
    if (raw_data)
    {
        var n = raw_data.size();
        var n2 = fhGetPow2(n);
        if (n2 > 8)
        {
            var c_data = raw_data.slice(0, n2-1);
            return c_data;
        }
    }
}

function fhHaning(raw_data)
{
    if (raw_data)
    {
        var nb = raw_data.size();
        var h_data = new Float64Vector(nb);
        var i;
        for(i = 0; i < nb; i++)
        {
            var value = raw_data.get(i)* 0.5*(1.0-Math.cos(2.0*Math.PI*i/nb));
            h_data.set(i, value);
        }
        return h_data;
    }
}

function fhRealFFT(data)
{
// extraction de la partie relle aprs une FFT
// cette fonction renvoie la partie relle dans un vecteur
    if (data)
    {
        var Nfft = data.size()/2;
        var Real = new Float64Vector(Nfft);
        var i;
        for(i = 0; i < Nfft; i++)
        {
            var v = data.get(i*2);
            Real.set(i, v);
        }
        return Real;
    }
}

function fhImgFFT(data)
{
// extraction de la partie relle aprs une FFT
// cette fonction renvoie la partie relle dans un vecteur
    if (data)
    {
        var Nfft = data.size()/2;
        var Img = new Float64Vector(Nfft);
        var i;
        for(i = 0; i < Nfft; i++)
        {
            Img.set(i, data.get(i*2 + 1));
        }
        return Img;
    }
}

function fhDsp(data, use_scale)
{
// calcul d'une densit spectrale
// en entre : Data contient les donnes (Float64Vector). Le nombre de donnes DOIT
// tre une puissance de 2 (aucune vrification  n'est faite ici).
// en retour revoie un vecteur correspondant  la densit spectrale
// de puissance.

    if (data)
    {
        var coeffs = jsFilter.FFTVector(data);
        if (coeffs)
        {
            var Real = fhRealFFT(coeffs);
            var Img = fhImgFFT(coeffs);
            if (Real && Img)
            {
                var n = Real.size();
                var dsp = new Float64Vector(n);
                var i;
                var max = -1.0;
                var t_max = 0;
                for(i = 0; i < n; i++)
                {
                    var d = Math.sqrt(Real.get(i)*Real.get(i) + Img.get(i)*Img.get(i));
                    if (d > max) 
                    {
                        max = d;
                        t_max = i;
                    }
                    dsp.set(i, d);
                }
                if (use_scale)
                {
                    dsp.scale(100.0/max);
                }
                
                return dsp;
            }
        }
    }
}
//------------------------------------
// Analyse Frquentiel
//------------------------------------
function fhDataToCoeff(Data, nbHar)
{
// Calcul des coefficients harmoniques et des cart-types associs.
// en entre : Data contient les donnes	       
//     nbhar est le nombre d'harmonique  calculer
// cette fonction revoie un tableau de deux colonnes
// la premire colonne correspond aux coefficients (2*NbHar +1)
// la deuxime  l'cart type par rapport au modle harmonique.
// exemple :
//           var harmo = fhDataToCoeff(Data, 100);	// calcul des 100 harmoniques
	if (Data)
    {
        var nbData = Data.size();
        if (nbHar <= 0 || nbHar > (nbData-1)/2) 
        {
            NbHar = (nbData-1)/2;
        }
        var nf = 2*nbHar+1;
        var Coeff = new Float64Vector(nf);			        // coefficient
        var Sd = new Float64Vector(nbHar+1);				// cart-type

        var n1 = nbData-1;
        var rm = nbData;
        var om = 2.0* Math.PI/rm;
        var sy = 0.0;
        var syq = 0.0;

        var i;
        for(i =0; i < n1; i++)
        {
            sy  = sy + Data.get(i);
            syq = syq+Data.get(i)*Data.get(i);
        }
        var cyy = syq - sy*sy/rm;
        var x = 0.0;
        var ss;
        Sd.set(0, Math.sqrt(cyy/(rm-1.0)));
        var j;
        for(j =0; j < nf; j++)
        {
            Coeff.set(j,0);
            var k = (j+1) / 2;
            for( i = 0; i < n1; i++)
            {
                if (k*2 != j+1)	x = Math.sin(om*i*k);		// valeur impaire 1, 3, 5 ...
                if (k*2 == j+1) x = Math.cos(om*i*k);       // valeur paire 2, 4, 6, 8...
                if (j == 0)		x = 0.5;				    // valeur moyenne -> 0
                Coeff.set(j, Coeff.get(j) + Data.get(i)*x);
            }
            Coeff.set(j, Coeff.get(j)*2.0/rm);
            if (j==0)
                ss = 0;
            else 
                ss = Coeff.get(j)*Coeff.get(j)*rm/2.0;

            cyy  = cyy-ss;
            if (k*2 != j+1 && j != 0 && j < n1-1)
            {
                sd.set(k, Math.sqrt(cyy/(rm-j+1)));
			}
        }
        var ret = new Array(Coeff, Sd);       
        return ret;
    }
}
//----------------------------------------------------------------
//      Filtre harmonique
//----------------------------------------------------------------
var freq_parameter_file_name = "harmo_filter.ini";        // fichier des paramtres
var freq_low = 0.5;
var freq_high = 15.0;
var freq_use = true;

function freq_harmo_filter_load_parameters()
{
    var config = new jsConfig(freq_parameter_file_name, "r");
    if (config.Open())
    {
        freq_low = config.Read("LOW", freq_low);
        freq_high = config.Read("HIGH", freq_high);
        freq_use = config.Read("USE", freq_use);
        config.Close();
    }
}

function freq_harmo_filter_save_parameters()
{
    var config = new jsConfig(freq_parameter_file_name, "w");
    if (config.Open())
    {
        config.Write("LOW", freq_low);
        config.Write("HIGH", freq_high);
        config.Write("USE", freq_use);
        config.Close();
    }
    else print("Impossible de sauver le fichier configuration");
}

function freq_harmo_filter_ask()
{
    var dialog, f1, f2;
    var w_height = 25;
    var w_space = 7;
    var w_pos = 20;
    var x_start = 10;
    var width = 120;
    freq_harmo_filter_load_parameters();
    
    dialog = new FLTKDialog(100, 100, 400, 150, "Filtre FFT pass Bande");        // cration de la boite de dialogue
    dialog.Center();                                                // centrer la boite de dialogue
    
    f1 = dialog.AddFloat(x_start, w_pos, width, w_height, freq_low);
    dialog.SetToolTip(f1, "frequence basse de coupure");
    dialog.AddStatic(x_start + width + w_space, w_pos, 100, 30, "Freq. Basse (Hz)");                     

    w_pos = w_pos + w_height + w_space;                                     // descendre la position courante
    f2 = dialog.AddFloat(x_start, w_pos, width, w_height, freq_high);
    dialog.SetToolTip(f2, "frequence haute de coupure");
    dialog.AddStatic(x_start + width + w_space, w_pos, 100, 30, "Freq. Haute (Hz)");                     

    w_pos = w_pos + w_height + w_space;                                     // descendre la position courante
    var c1 = dialog.AddCheckBox(x_start, w_pos, width, w_height, " Utiliser", freq_use);

    var ret = dialog.Execute();                                             // excuter la boite de dialogue
    if (ret == 1)
    {
        var  fc1 = dialog.GetValue(f1);
        var  fc2 = dialog.GetValue(f2);
        if (fc1 > 0 && fc2 > fc1)
        {
            freq_low = fc1;
            freq_high = fc2;
            freq_use = dialog.GetBoolValue(c1);
        }
        else ret = 0;
    }
    return ret;
}

function freq_harmo_filter(raw_data, rate)
{
    if (raw_data)
    {
        var i;
        var size = raw_data.size();
        var coeffs = jsFilter.FFTVector(raw_data);
        var size_2 = coeffs.size();
        var d_size = size_2/2;
        var d_rate = 2*d_size/(rate);           
        if (freq_use)
        {
            var u1 = Math.round(freq_low*d_rate);
            var u2 = Math.round(freq_high*d_rate);
            for(i = 0; i < u1; i++)
            {
                var idx = i*2;
                coeffs.set(idx, 0);
                coeffs.set(idx+1, 0);
            }
            for(i = u2; i < size_2; i++)
            {
                var idx = i*2;
                coeffs.set(idx, 0);
                coeffs.set(idx+1, 0);
            }
        }
        var tmp = jsFilter.InvFFTVector(coeffs);
        var size = tmp.size();
        var new_data = new Float64Vector(size); 
        for(i = 0; i < size; i++)
            new_data.set(i, tmp.get(i));
        return new_data;
    }
    return undefined;
}
