//--------------------------------------------------------------------------
//  Objet analog : utilisation des vecteurs et non des Array
//  Gilles Dietrich, 2015
//--------------------------------------------------------------------------
function jsAnalog64(name_or_id)
{
    this.name = "";                             // nom du marqueur
    this.id = -1;                               // numro [0..num-1]
    this.Data = undefined;;                     // donnes
    this.isOk = false;                          // flag
    this.frames = Mi.GetAnalogFrameCount();
    this.rate = Mi.GetAnalogRate();
    this.rate_ratio = this.rate/Mi.GetRate();
    var max_id = Mi.GetAnalogCount();
    if (typeof(name_or_id) == "number")         //creation avec un numro [0..N-1]
    {
        if (name_or_id >=0 && name_or_id < max_id)
        {
            this.id = name_or_id;
            this.name = Mi.GetAnalogName(this.id);
        }
    }
    else if (typeof(name_or_id) == "string")    // creation avec le nom
    {
        this.name = name_or_id;
        this.id = Mi.GetAnalogNum(this.name);
    }

    this.Load();                                // chargement des donnes        
}

jsAnalog64.prototype.Load = function()
{
    var max_id = Mi.GetAnalogCount();
    this.isOk = false;
    if (this.id >= 0 && this.id < max_id && this.frames > 0)
    {
        this.Data = Mi.GetAnalogDataBuffer64(this.id);
        if (this.Data)
        {
            this.isOk = true;
            this.frames = Mi.GetAnalogFrameCount();
            this.rate = Mi.GetAnalogRate();
            this.rate_ratio = this.rate/Mi.GetRate();
        }
    }
    return this.isOk;
}

jsAnalog64.prototype.SetElement = function(frame, value)
{
    var ret = false;
    if (this.isOk && typeof(frame) == "number" && typeof(value) == "number")
    {
        if (frame >=0 && frame < this.frames)
        {
            ret = true;
            this.Data.set(frame, value);
        }
    }
    return ret;
}

jsAnalog64.prototype.GetElement = function(frame)
{
    var ret = 0;
    if (this.isOk && typeof(frame) == "number")
    {
        if (frame >=0 && frame < this.frames)
        {
            ret = this.Data.get(frame);
        }
    }
    return ret;
}


jsAnalog64.prototype.Scale = function(scale)
{
    var ok = false;
    if (this.isOk && typeof(scale) == "number")
    {
        this.Data.scale(scale);
        ok = true;
    }
    return ok;
}

jsAnalog64.prototype.Save = function()
{
    var ok = false;
    if (this.isOk)
    {
        Mi.PutAnalogDataBuffer64(this.id, this.Data);
        ok = true;
    }
    return ok
}

jsAnalog64.prototype.SaveAs = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetAnalogNum(name);
        if (new_id < 0) ok = this.Create(name);
        else
        {
            this.id = new_id;
            this.name = name;
            ok = this.Save();
        }
    }
    return ok;
}

jsAnalog64.prototype.Create = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetAnalogNum(name);
        if (new_id < 0)
        {
            Mi.NewAnalog(name);
            new_id = Mi.GetAnalogNum(name);
            if (new_id >= 0)
            {
                this.id = new_id;
                this.name = name;
                ok = this.Load();
            }
        }
    }
    return ok;
}

jsAnalog64.prototype.Copy = function(name)
{
    var ok = false;
    var new_id = -1;
    var max_id = Mi.GetAnalogCount();
    if (typeof(name) == "string")
        new_id = Mi.GetAnalogNum(name);
    else if (typeof(name) == "number")
        new_id = name;
    if (new_id >=0 && new_id < max_id)
    {
        var old_id = this.id;
        this.id = new_id;
        ok = this.Load();
        this.id = old_id;
    }
    return ok;
}

//----------------------------------------------------------------------------------
function jsAnalogExist(name)
{
    var ok = false;
    var id = -1;
    if (typeof(name) == "string")
        id = Mi.AnalogNum(name);
    else if (typeof(name) == "number")
        id = name;
    if (id >=0 && id < Mi.AnalogCount())
        ok = true;
    return ok;
}

//
//  Ajout de fonction Marker  l'objet Mi
//
Mi.AnalogExists = jsAnalogExist;
