//--------------------------------------------------------------------------------
//  MatLab wrapper
//--------------------------------------------------------------------------------
function jsMatlab (matlab_path)
{
    if (matlab_path == undefined)
    {
        this.path = "";
    }
    else
        this.path = matlab_path;
    this.is_ok = false;
    this.debug = false;
    this.graph_type = "plot";
    this.plot_color = "black";
    this.LineWidth = 1;
}

jsMatlab.prototype.set_default = function()
{
    this.graph_type = "plot";
    this.plot_color = "black";
    this.LineWidth = 1;
    this.set_grid(false);
}

jsMatlab.prototype.Open = function()
{
    this.is_ok = MatLab.Open(this.path);
    return this.is_ok;
}

jsMatlab.prototype.Close = function()
{
    MatLab.Close();
}

jsMatlab.prototype.IsOpen = function()
{
    return MatLab.IsOpen();
}

jsMatlab.prototype.Eval = function(cmd)
{
    var ret = undefined;
    if (this.is_ok)
    {
        ret = MatLab.Eval(cmd);
        if (this.debug)
            print(cmd+" > "+ret);
    }
    return ret;
}

jsMatlab.prototype.HoldOn = function(hold_on)
// set plot or replot mode
{
    if (hold_on)
        this.Eval("hold on;");
    else
        this.Eval("hold off;");
}

gnu_plot.prototype.win_title = function(title)
{
    if (this.is_ok && title)
    {
        var cmd = "gcf,'name','"+ title + "','numbertitle','off');";
        this.Eval(cmd);
    }
}

jsMatlab.prototype.NewFigure = function(title)
// open a new empty window
{
    if (this.is_ok)
    {
        this.Eval("figure;");
        this.win_title(title);
    }
}

jsMatlab.prototype.put_vector = function(vec, name)
{
    if (this.is_ok)
    {
        return MatLab.PutVector(vec, name);
    }
    return -1;
}

jsMatlab.prototype.check_path = function(path_name)
{
    var new_path = path_name;
    if (typeof(path_name) == "string")
    {
        new_path = "";
        var i;
        var n = path_name.length;
        for(i = 0; i < n; i++)
        {
            if (path_name[i] == "\\")
                new_path = new_path + '/';
            else
                new_path = new_path + path_name[i];
        }
    }
    return new_path;
}

jsMatlab.prototype.save_matrix = function(data)
{
    if (data)
    {
        var path_name = Mi.GetTmpDirectory() + "\\mat_tmp_data.tmp";
        path_name = this.check_path(path_name);
        var tmp_file = new File(path_name, "w");
        if (r_file.isOpen())
        {            
            var i, j;
            var nb_col = data.length;
            var nb_lig = data[0].size();
            for(i = 0; i < nb_lig; i++)
            {
                var st = "";
                for(j = 0; j < nb_col; j++)
                {
                    var v = data[j];
                    var value = v.get(i);
                    st = st+ value.toFixed(this.Precision);
                    if (j < nb_col -1) st = st + "  ";
                }
                r_file.writeLine(st);                    
            }
            r_file.close();
            return path_name;
        }
    }
    return undefined;
}

jsMatlab.prototype.save_array = function(data)
{
    if (data)
    {
        var path_name = Mi.GetTmpDirectory() + "\\mat_tmp_data.tmp";
        path_name = this.check_path(path_name);
        var tmp_file = new File(path_name, "w");
        if (r_file.isOpen())
        {            
            var i, j;
            var nb_col = data.length;
            var nb_lig = data[0].length;
            for(i = 0; i < nb_lig; i++)
            {
                var st = "";
                for(j = 0; j < nb_col; j++)
                {
                    var value = data[j][i];
                    st = st+ value.toFixed(this.Precision);
                    if (j < nb_col -1) st = st + "  ";
                }
                r_file.writeLine(st);                    
            }
            r_file.close();
            return path_name;
        }
    }
    return undefined;
}

jsMatlab.prototype.put_array = function(data, mat_name)
{
    if (this.is_ok && data)
    {
        var path_name = save_array(data);
        if (data)
        {
            var cmd = mat_name+"=load('"+path_name+"','-ascii');";            
            this.Eval(cmd);
            return true;
        }
    }
    return false;
}

jsMatlab.prototype.put_matrix = function(data, mat_name)
{
    if (this.is_ok && data)
    {
        var path_name = save_matrix(data);
        if (data)
        {
            var cmd = mat_name+"=load('"+path_name+"','-ascii');";            
            this.Eval(cmd);
            return true;
        }
    }
    return false;
}

jsMatlab.prototype.title = function(title, font, size)
{
    if (this.is_ok && title)
    {
        var cmd = "title('"+title;
        if (typeof(size) == "number")
            cmd=cmd+",'FontSize',"+size;
        if (typeof(font) == "string")
            cmd = cmd + ",'FontName','"+font+"'";
        cmd  = cmd + ");";
        return this.Eval(cmd);
    }
    return false;
}

jsMatlab.prototype.xlabel = function(title, font, size)
{
    if (this.is_ok && title)
    {
        var cmd = "xlabel('"+title;
        if (typeof(size) == "number")
            cmd=cmd+",'FontSize',"+size;
        if (typeof(font) == "string")
            cmd = cmd + ",'FontName','"+font+"'";
        cmd  = cmd + ");";
        return this.Eval(cmd);
    }
    return false;
}

jsMatlab.prototype.ylabel = function(title, font, size)
{
    if (this.is_ok && title)
    {
        var cmd = "ylabel('"+title;
        if (typeof(size) == "number")
            cmd=cmd+",'FontSize',"+size;
        if (typeof(font) == "string")
            cmd = cmd + ",'FontName','"+font+"'";
        cmd  = cmd + ");";
        return this.Eval(cmd);
    }
    return false;
}

jsMatlab.prototype.zlabel = function(title, font, size)
{
    if (this.is_ok && zlabel)
    {
        var cmd = "zlabel('"+title;
        if (typeof(size) == "number")
            cmd=cmd+",'FontSize',"+size;
        if (typeof(font) == "string")
            cmd = cmd + ",'FontName','"+font+"'";
        cmd  = cmd + ");";
        return this.Eval(cmd);
    }
    return false;
}

jsMatlab.prototype.axis_2D = function(xmin, xmax, ymin ,ymax, title, xlabel, ylabel)
{
    if (this.is_ok)
    {
        if (title) this.title(title);
        if (xlabel) this.xlabel(xlabel);
        if (ylabel) this.ylabel(ylabel);
        var cmd = "axis["+xmin+" "+xmax+" "+ymin+" "+ymax+"];";
        return this.Eval(cmd);
    }
}

jsMatlab.prototype.plot_2d = function(x_data, y_data, title, xlabel, ylabel)
{
    if (this.is_ok && x_data && y_data)
    {
        if (title) this.title(title);
        if (xlabel) this.xlabel(xlabel);
        if (ylabel) this.ylabel(ylabel);
        MatLab.PutVector(x_data, "x");
        MatLab.PutVector(y_data, "y");
        var cmd  = this.graph_type + "(x,y";
        cmd = cmd + ",'color,'"+this.plot_color+"'";
        cmd = cmd + ",'LineWidth',"+this.LineWidth;
        cmd = cmd + ");";
        return this.Eval(cmd);
    }
}

jsMatlab.prototype.plot_1d = function(rate, y_data, title, xlabel, ylabel)
{
    if (this.is_ok && y_data)
    {
        var n = y_data.size();
        var time = new Float64Vector(n);
        var i;
        for(i = 0; i < n; i++)
        {
            var t = i/rate;
            time.set(i, t);
        }
        return this.plot_2d(time, y_data, title, xlabel, ylabel);
    }
    return false;
}

jsMatlab.prototype.set_grid  = function(grid_on)
{
    if (this.is_ok)
    {
        var cmd = "grid ";
        if (grid_on) cmd = cmd + "on;";
        else cmd = cmd + "off;";
        return this.Eval(cmd);
    }
}

jsMatlab.prototype.plot_point = function(x, y)
{
    if (this.is_ok)
    {
        this.Eval("hold on;");
        var cmd = "plot("+x+","+y+","+"'ko');";
        return this.Eval(cmd);
    }
}

jsMatlab.prototype.plot_points = function(x, y)
{
    if (this.IsOpen && x!= undefined && y != undefined)
    {
        var len, i;
        if (x.size == undefined)
        {
            len = x.length;
            for(i = 0; i < len; i++)
            {
                this.plot_point(x[i], y[i]);
            }
        }
        else
        {
            len = x.size();
            for(i = 0; i < len; i++)
            {
                this.plot_point(x.get(i), y.get(i));
            }
        }
        return true;
    }
    return false;
}


jsMatlab.prototype.save_plot = function(filename, type)
// hgexport(gcf, 'figure1.jpg', hgexport('factorystyle'), 'Format', 'jpeg');
{
    var	SaveType = ["fig", "m", "jpg", "eps", "pdf", "bmp", "emf", "pbm", "pcx", "ppm", "tif"];
    var Type = "emf";
    if (this.IsOpen && filename)
    {
        if (typeof(type) == "string")
        {
            var size = SaveType.length;
            var i;
            var t = -1;
            for(i = 0; i < size; i++)
            {
                if (type == SaveType[i]) 
                    t = i;
            }
            if (t >=0) Type = type;
            else print("saveplot : using default format " + Type);
        }
        var Path = this.CheckPath(filename) + "."+Type;
        var cmd =  "saveas(gcf,'"+Path+"');";
        return this.Eval(cmd);
    }
    return false;
}

// "GetVector"
// "GetVector64"