require("jsgnuplot")
require("jsR")
//require("jsMatlab")
//------------------------------------------------------------------------------
//  wapper graphique pour R, gnuplot, matlab 
//------------------------------------------------------------------------------
function jsPlot()
{
    this.use_active_x = false;                              // use active X or dll
    this.hold_on = false;                                   // 
    this.gnuplot = undefined;
    this.R = undefined;
    this.matlab = undefined;
    this.driver = undefined;
    this.is_open = false;
    this.debug = false;
}

jsPlot.prototype.Open = function(driver)
{
    if (this.is_open) 
    {
        print("driver "+this.driver+" is already opened.");
        return true;
    }
    if (driver == "R")                                      // try to open R
    {
        print("Trying ro open R driver...");
        this.R = new jsR_project(this.use_active_x);        // open R with activex or use DLL directly
        this.R.debug = this.debug;
        this.is_open = this.R.Open();                       // open R
        if (this.is_open) this.driver = "R";
        else this.R = undefined;
    }
    else if (driver == "matlab")
    {
        print("Trying ro open MatLab driver...");
        print("Not implemented yet...");
    }
    else                                                    // by default, try to open gnuplot
    {
        print("Trying to open GnuPlot driver...");
        this.gnuplot = new gnu_plot();                      // open gnuplot process
        this.gnuplot.debug = this.debug;
        this.is_open =  this.gnuplot.Open();                // and try to connect
        if (this.is_open) this.driver = "gnuplot";
        else this.gnuplot = undefined;
    }
    if (!this.is_open) print("Driver <"+driver+"> not found");
    return this.is_open;
}


jsPlot.prototype.Close = function()
{
    if (this.is_open)
    {
        if (this.gnuplot) this.gnuplot.Close();
        else if (this.R) this.R.Close();
        //else if (this.matlab)
        //this.gnuplot = undefined;
        this.R = undefined;
        this.matlab = undefined;
        this.driver = undefined;
        this.is_open = false;
        return true;
    }
    return false;
}

jsPlot.prototype.HoldOn = function(hold_on)
{
    if (this.is_open)
    {
        if (this.gnuplot) this.gnuplot.HoldOn(hold_on);
        else if (this.R) this.R.HoldOn = hold_on;
        //else if (this.matlab)
        return true;
    }
    return false;
}

jsPlot.prototype.NewFigure = function(title)
{
    if (this.is_open)
    {
        if (this.gnuplot) this.gnuplot.NewFigure(title);
        else if (this.R) this.R.NewFigure(title);
        //else if (this.matlab)
    }
    return false;
}

jsPlot.prototype.plot_2d = function(x_data, y_data, title, xlabel, ylabel)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            if (!this.gnuplot.hold_on)
            {
                this.NewFigure(title);
                this.gnuplot.Send("set autoscale");
            }
            this.gnuplot.plot_2d(x_data, y_data, title, xlabel, ylabel);
            this.gnuplot.refresh();
            return true;
        }
        else if (this.R)
        {
            if (xlabel == undefined) xlabel = "";
            if (ylabel == undefined) ylabel = "";
            this.R.Plot2DAuto(x_data, y_data, title, xlabel, ylabel);
            return true;
        }
        else if (this.matlab)
        {
        }
    }
    return false;
}

jsPlot.prototype.plot_1d = function(rate, y_data, title, xlabel, ylabel)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            if (!this.gnuplot.hold_on)
            {
                this.NewFigure(title);
                this.gnuplot.Send("set autoscale");
            }
            this.gnuplot.plot_1d(rate, y_data, title, xlabel, ylabel);
            this.gnuplot.refresh();
            return true;
        }
        else if (this.R)
        {
            if (xlabel == undefined) xlabel = "";
            if (ylabel == undefined) ylabel = "";
            this.R.Plot1D(y_data, rate, title, xlabel, ylabel);
            return true;
        }
        else if (this.matlab)
        {
        }        
    }
    return false;
}

jsPlot.prototype.plot_point = function(x, y)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            return this.gnuplot.plot_point(x, y);
        }
        else if (this.R)
        {
            return this.R.plot_point(x, y);
        }
        else if (this.matlab)
        {
        }
    }
    return false;
}

jsPlot.prototype.plot_points = function(x, y)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            return this.gnuplot.plot_points(x, y);
        }
        else if (this.R)
        {
            return this.R.plot_points(x, y);
        }
        else if (this.matlab)
        {
        }
    }
    return false;
}

jsPlot.prototype.save_plot = function(filename, ext)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            return this.gnuplot.save_plot(filename, ext);
        }
        else if (this.R)
        {
            return this.R.SavePlot(filename, ext);
        }
        else if (this.matlab)
        {
        }
    }
    return false;
}

jsPlot.prototype.send = function(cmd)
{
    if (cmd && this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.Send(cmd);
        }
        else if (this.R)
        {
            this.R.Send(cmd);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.plot_line = function(x1, y1, x2, y2)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.plot_line(x1, y1, x2, y2);
        }
        else if (this.R)
        {
            this.R.plot_line(x1, y1, x2, y2);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.draw_x_axis = function()
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.draw_x_axis();
        }
        else if (this.R)
        {
            this.R.draw_x_axis();
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.draw_y_axis = function()
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.draw_y_axis();
        }
        else if (this.R)
        {
            this.R.draw_y_axis();
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.title = function(title, font, size)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.title(title, font, size);
        }
        else if (this.R)
        {
            if (size == undefined) size = 1;
            else size = size/12;
            this.R.title(title, 1, 1);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.xlabel = function(label, font, size)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.xlabel(label, font, size);
        }
        else if (this.R)
        {
            this.R.xlabel(label, 1, size/12.0);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.ylabel = function(label, font, size)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.ylabel(label, font, size);
        }
        else if (this.R)
        {
            this.R.ylabel(label, 1, size/12.0);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.win_title = function(title)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.win_title(title);
        }
        else if (this.R)
        {
            this.R.win_title(title);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.reset = function()
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.reset();
        }
        else if (this.R)
        {
            this.R.reset();
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.set_line_type = function(line_type)
{
   if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.dt = line_type;
        }
        else if (this.R)
        {
            //this.R.reset();
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.set_line_color = function(line_color)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.color = 'rgbcolor ' +  '"' + line_color + '"';
        }
        else if (this.R)
        {
            this.R.color = line_color;
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.set_line_width = function(line_width)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.lw = line_width;
        }
        else if (this.R)
        {
            //this.R.reset();
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.set_grid = function(grid_on)
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.grid = grid_on;
        }
        else if (this.R)
        {
            this.R.set_grid(grid_on);
        }
        else if (this.matlab)
        {
        }
    }
}

jsPlot.prototype.refresh = function()
{
    if (this.is_open)
    {
        if (this.gnuplot) 
        {
            this.gnuplot.refresh();
        }
        else if (this.R)
        {
            //this.R.reset();
        }
        else if (this.matlab)
        {
        }
    }
}

