//-----------------------------------------------------------
//  Stat connector for r-project
//-----------------------------------------------------------
function jsRConnector(use_active_x)
{

    this.Server = undefined;
    if (use_active_x)               // start active X DCOMM server if required
        this.Server =  new ActiveXObject("StatConnectorSrv.StatConnector");
    else R.Open();                  // or use R library directly
    
    this.Eror = 0;
    this.debug = false;
}

jsRConnector.prototype.IsOk = function()
{
    this.Eror = 0;
    if (this.Server)
    {
        if (this.Server.IsOk()) return true;
    }
    else if (R.IsOpen()) return true;
    this.Error = -1;
    return false;
}

jsRConnector.prototype.Init = function(connector_name)
{
    if (this.Server)
    {
        if (this.Server.IsOk())
        {
            this.Server.callMethod("Init", connector_name);
            return true;
        }
    }
    else return R.IsOpen();
    return false;
}

jsRConnector.prototype.Close = function()
{
    if (this.Server)
    {
        if (this.Server.IsOk())
        {
            this.Server.callMethod("Close");
            return true;
        }
    }
    return R.Close();
}

jsRConnector.prototype.Evaluate = function(expression)
{
    var ret =  undefined;
    if (this.debug)
        write(expression + "\n");
    if (this.Server)
    {
        if (this.Server.IsOk())
        {
            ret = this.Server.callMethod("Evaluate", expression);
        }
    }
    else ret = R.Eval(expression)
    return ret;
}

jsRConnector.prototype.EvaluateNoReturn = function(expression)
{
    if (this.debug)
        write(expression + "\n");
    if (this.Server)
    {
        if (this.Server.IsOk())
        {
            this.Server.callMethod("EvaluateNoReturn", expression);
            return true;
        }
        return false;
    }
    return R.Eval(expression);
}
//-----------------------------------------------------------
//  r-project wrapper
//-----------------------------------------------------------
function jsR_project(use_active_x)
{
    this.stat_connector = new jsRConnector(use_active_x);
    this.IsOpen = false;
    this.HoldOn = false;
    this.GraphType = 0;     //0: normal, 1:xLog, 2:yLog , 3:LogLog
    this.Symbol = "\"l\"";
    this.Precision = 5;
    this.debug = false;
    this.grid_on = false;
    this.lineType = 1;
    this.lineWidth = 1; 
    this.textSize = 1;
    this.color = 1;
}

jsR_project.prototype.reset = function()
{
    this.HoldOn = false;
    this.GraphType = 0;     //0: normal, 1:xLog, 2:yLog , 3:LogLog
    this.Symbol = "\"l\"";
    this.Precision = 5;
    this.debug = false;
    this.grid_on = false;
    this.lineType = 1;
    this.lineWidth = 1; 
    this.textSize = 1;
    this.color = 1;
}

jsR_project.prototype.Open = function()
{
    if (this.IsOpen) return true;
    if (this.stat_connector.IsOk())
    {
        this.stat_connector.debug = this.debug;
        this.IsOpen = this.stat_connector.Init("R");
    }
    return this.IsOpen;
}

jsR_project.prototype.Close = function()
// close main application with R
{
    if (this.IsOpen && this.stat_connector.IsOk())
    {
        this.stat_connector.EvaluateNoReturn("dev.off()");
        this.stat_connector.Close();
        this.IsOpen = false;
        this.stat_connector = undefined;
    }
}

jsR_project.prototype.Send = function(cmd)
{
    if (this.IsOpen)
        this.stat_connector.EvaluateNoReturn(cmd);
}

jsR_project.prototype.win_title = function(title)
{
    if (this.IsOpen && title != undefined)
    {
        //var cmd = "dev.new(title='" + title + "')";
        var cmd = "dev.cur(title='" + title + "')";
        this.stat_connector.EvaluateNoReturn(cmd);
    }
}

jsR_project.prototype.NewFigure = function(title)
{
    var ret = undefined;
    if (this.IsOpen)
    {
        if (title != undefined)
        {
            var cmd = "dev.new(title='" + title + "')";
            this.stat_connector.EvaluateNoReturn(cmd);
            ret = this.stat_connector.Evaluate("dev.cur()");
        }
        else
        {
            this.stat_connector.EvaluateNoReturn("win.graph()");
            this.stat_connector.EvaluateNoReturn("frame()");
            ret = this.stat_connector.Evaluate("dev.cur()");
        }
    }
    return ret;
}

jsR_project.prototype.CloseFigure = function()
{
    if (this.IsOpen)
        this.stat_connector.EvaluateNoReturn("dev.off()");
}

jsR_project.prototype.CreateMatrix = function(col, lig)
{
    if (typeof(col) != "number") return undefined;
    if (typeof(lig) != "number") return undefined;

    var data = new Array(col);
    var i, j;
    for(i = 0; i < col; i++)
    {
        data[i] = new Array(lig);
    }
    for(i = 0; i < col; i++)
    {
        for(j = 0; j < lig; j++)
            data[i][j] = 0.0;
    }
    return data;
}

jsR_project.prototype.CheckPath = function(path_name)
{
    var new_path = path_name;
    if (typeof(path_name) == "string")
    {
        new_path = "";
        var i;
        var n = path_name.length;
        for(i = 0; i < n; i++)
        {
            if (path_name[i] == "\\")
                new_path = new_path + '/';
            else
                new_path = new_path + path_name[i];
        }
    }
    return new_path;
}

jsR_project.prototype.PutVector = function(vec, vec_name)
{
    if (this.IsOpen && vec)
    {
        var path_name = Mi.GetTmpDirectory() + "\\rv_tmp_data.txt";
        path_name = this.CheckPath(path_name);
        var file = new File(path_name, "wb");
        if (file.isOpen())
        {
            file.close();
            if (vec.size == undefined)
            {
                var len = vec.length;
                var i;
                for(i = 0; i < len; i++)
                {
                    var f_tmp = new File(path_name, "wa");
                    var value = vec[i];
                    f_tmp.writeLine(value.toFixed(this.Precision));
                    f_tmp.close();
                }
            }
            else
            {
                var len = vec.size();
                var i;
                for(i = 0; i < len; i++)
                {
                    var f_tmp = new File(path_name, "wa");
                    var value = vec.get(i);
                    f_tmp.writeLine(value.toFixed(this.Precision));
                    f_tmp.close();
                }
            }
            
            var cmd = vec_name+"<-read.table(\""+ path_name + "\")";            
            this.stat_connector.EvaluateNoReturn(cmd);
            return true;
        }
    }
    return false;
}

jsR_project.prototype.PutArray = function(data, mat_name)
{
    if (this.IsOpen && data)
    {
        var path_name = Mi.GetTmpDirectory() + "\\rm_tmp_data.txt";
        path_name = this.CheckPath(path_name);
        var r_file = new File(path_name, "w");
        if (r_file.isOpen())
        {            
            var i, j;
            var nb_col = data.length;
            var nb_lig = data[0].length;
            for(i = 0; i < nb_lig; i++)
            {
                var st = "";
                for(j = 0; j < nb_col; j++)
                {
                    var value = data[j][i];
                    st = st+ value.toFixed(this.Precision);
                    if (j < nb_col -1) st = st + "  ";
                }
                r_file.writeLine(st);                    
            }
            r_file.close();
            var cmd = mat_name+"<-read.table(\""+ path_name + "\")";            
            this.stat_connector.EvaluateNoReturn(cmd);
            return true;
        }
    }
    return false;
}

jsR_project.prototype.PutMatrix = function(data, mat_name)
{
    if (this.IsOpen && data)
    {
        if (data[0].size == undefined)
        {
            return this.PutArray(data, mat_name);
        }
        var path_name = Mi.GetTmpDirectory() + "\\mat_tmp_data.txt";
        path_name = this.CheckPath(path_name);
        var r_file = new File(path_name, "wt");
        if (r_file.isOpen())
        {            
            var i, j;
            var sv = "";
            var nb_lig = data[0].size();
            var nb_col = data.length;
            for(i = 0; i < nb_lig; i++)
            {
                sv = "";
                for(j = 0; j < nb_col; j++)
                {
                    var v = data[j];
                    var value = v.get(i);
                    sv = sv + value.toFixed(this.Precision);
                    if (j < nb_col -1) sv = sv + " ";
                }                    
                r_file.writeLine(sv);                    
            }
            r_file.close();
            var cmd = mat_name+"<-read.table(\""+ path_name + "\")";            
            this.stat_connector.EvaluateNoReturn(cmd);
            return true;
        }
    }
    return false;
}

jsR_project.prototype.set_title = function(what, title, font, size)
{
        var cmd = "title("+ what+" = '"+ title+"'";
        if (font != undefined)
            cmd = cmd + ", font."+ what +" = "+font; 
        if (size != undefined)
            cmd = cmd + ", cex."+ what +" = "+size; 
        cmd = cmd+")";
        this.stat_connector.EvaluateNoReturn(cmd);        
}

jsR_project.prototype.title = function(label, font, size)
{
    if (this.IsOpen)
    {
        this.set_title("main", label, font, size);
    }
}

jsR_project.prototype.xlabel = function(label, font, size)
{
    if (this.IsOpen)
    {
        this.set_title("xlab", label, font, size);
    }
}

jsR_project.prototype.ylabel = function(label, font, size)
{
    if (this.IsOpen)
    {
        this.set_title("ylab", label, font, size);
    }
}

jsR_project.prototype.Axis2D = function(xmin, xmax, ymin ,ymax, Title, XLabel, YLabel)
// Draw 2 axes (X and Y) with title and labels
{
    if (this.IsOpen)
    {
        var axis = this.CreateMatrix(2, 2);
        var X = new Array(2);
        X[0] = xmin;
        X[1] = xmax;
        var Y = new Array(2);
        Y[0] = ymin;
        Y[1] = ymax;
        var M = new Array(2);
        M[0] = X;
        M[1] = Y;
        
        this.PutMatrix(M, "a");
        var cmd;
        if (this.HoldOn)
            cmd= "lines(a, type=\"n\"";
        else
            cmd= "plot(a, type=\"n\"";
        if (typeof(this.textSize) =="number")
        {
            cmd = cmd + ", cex.axis = "+ this.textSize.toString();
            cmd = cmd + ", cex.lab = "+ this.textSize.toString();
            cmd = cmd + ", cex.main = "+ this.textSize.toString();
        }
        if (typeof(XLabel) == "string")
            cmd = cmd + ', xlab="' + XLabel  + '"';
        if (typeof(YLabel) == "string")
            cmd = cmd + ', ylab="' + YLabel  + '"';
        if (typeof(Title) == "string")
            cmd = cmd + ', main="' + Title  + '"';
        cmd = cmd + ")";
        this.stat_connector.EvaluateNoReturn(cmd);        
        return true;
    }
    return false;
}

jsR_project.prototype.SavePlot = function(filename, type)
// Save current graphics to file 'Name' (with no extension)
// type could be : "wmf", "emf", "png", "jpeg", "jpg", "bmp", "ps", "eps", "pdf"
{
    var	rSaveType = ["wmf", "emf", "png", "jpeg", "jpg", "bmp", "ps", "eps", "pdf"];
    var Type = "emf";
    if (typeof(type) == "string")
    {
        var size = rSaveType.length;
        var i;
        var t = -1;
        for(i = 0; i < size; i++)
        {
            if (type == rSaveType[i]) t = i;
        }
        if (t >=0) Type = type;
        else print("saveplot : using default format " + Type);
    }
    var Path = this.CheckPath(filename) + "."+Type;
    if (this.debug)
        print("Saving : "+Path);
    
	var cmd = "savePlot(\""+ Path + "\", type=c(\""+ Type + "\"))";
    this.stat_connector.EvaluateNoReturn(cmd);        
    return true;
}

jsR_project.prototype.Plot2D = function(x, y)
//	Trace 2D: x et y sont des vecteurs
{
    if (this.IsOpen && x && y)
    {
        var cmd = "";
        var M = new Array(2);
        M[0] = x;
        M[1] = y;
        this.PutMatrix(M, "m");
        if (this.HoldOn)
        {
            cmd = "lines(m, type=" + this.Symbol;
        }
        else
        {
            cmd = "plot(m, type=" + this.Symbol;
        }
        
        if (typeof(this.color) =="number")
        {
            cmd = cmd + ", col="+this.color;
        }
        
        if (typeof(this.lineType) =="number")
        {
            cmd = cmd + ", lty="+this.lineType.toString();
        }
        if (typeof(this.lineWidth) =="number")
        {
            cmd = cmd + ", lwd="+this.lineWidth.toString();
        }
        if (typeof(this.textSize) =="number")
        {
            cmd = cmd + ", cex.axis = "+ this.textSize.toString();
            cmd = cmd + ", cex.lab = "+ this.textSize.toString();
            cmd = cmd + ", cex.main = "+ this.textSize.toString();
        }

        cmd = cmd + ")";
        this.stat_connector.EvaluateNoReturn(cmd);     

        return true;
    }
    return false;
}

jsR_project.prototype.Plot2DAuto = function(x, y, title, label_x, label_y)
{
    if (this.IsOpen && x &&  y)
    {
        var nx = x.size();
        var ny = y.size();
        if (nx == ny)
        {
            var max_x = x.get(0);
            var min_x = x.get(0);
            var max_y = y.get(0);
            var min_y = y.get(0);
            var i;
            for(i = 0; i < nx; i++)
            {
                var vx = x.get(i);
                var vy = y.get(i);
                if (vx > max_x) max_x = vx;
                if (vx < min_x) min_x = vx;
                if (vy > max_y) max_y = vy;
                if (vy < min_y) min_y = vy;
            }
            if (!this.HoldOn)
            {
                this.NewFigure(title);
                this.Axis2D(min_x, max_x, min_y, max_y, title, label_x, label_y);
                this.HoldOn = true;
            }
            
            this.Plot2D(x, y);
            return true;
        }
    }
    return false;
}

jsR_project.prototype.Plot1D = function(x, rate, title, label_x, label_y)
{
    if (this.IsOpen && x && rate > 0)
    {
        var n = x.size();
        var time = new Float64Vector(n);
        var i;
        var max_value = x.get(0);
        var min_value = x.get(0);
        for(i = 0; i < n; i++)
        {
            var value = x.get(i);
            if (value > max_value) max_value = value;
            if (value < min_value) min_value = value;
            time.set(i, i/rate);
        }
        var min_time = 0;
        var max_time = time.get(n-1); 
        if (!this.HoldOn)
        {
            this.NewFigure(title);
            this.Axis2D(min_time, max_time, min_value, max_value, title, label_x, label_y);
            this.HoldOn = true;
        }
        this.Plot2D(time, x);
        return true;
    }
    return false;
}

jsR_project.prototype.plot_point = function(x, y, pt_type)
/*
pt_type : 0:18: S-compatible vector symbols.
19: solid circle,
20: bullet (smaller solid circle, 2/3 the size of 19),
21: filled circle,
22: filled square,
23: filled diamond,
24: filled triangle point-up,
25: filled triangle point down.
32:127: ASCII characters

*/
{
    if (this.IsOpen && typeof(x) == "number" && typeof(y) == "number")
    {
        var pch = 16;
        if (typeof(pt_type) == "number") pch = pt_type;
        var cmd = "points("+x+" , "+y+" , pch = "+pch+")"
        this.stat_connector.EvaluateNoReturn(cmd);        
        return true;
    }
    return false;
}

jsR_project.prototype.plot_points = function(x, y, pt_type)
{
    if (this.IsOpen && x!= undefined && y != undefined)
    {
        var len, i;
        if (x.size == undefined)
        {
            len = x.length;
            for(i = 0; i < len; i++)
            {
                this.plot_point(x[i], y[i], pt_type);
            }
        }
        else
        {
            len = x.size();
            for(i = 0; i < len; i++)
            {
                this.plot_point(x.get(i), y.get(i), pt_type);
            }
        }
        return true;
    }
    return false;
}

jsR_project.prototype.set_grid = function(grid_on)
{
    if (this.IsOpen)
    {
        this.grid_on = grid_on;
        if (this.grid_on)
        {
            var cmd = "grid()";
            this.stat_connector.EvaluateNoReturn(cmd);        
        }
    }
}

jsR_project.prototype.plot_line = function(x1, y1, x2, y2)
{
    if (this.IsOpen && x1 != undefined && y1 != undefined && x2 != undefined && y2 != undefined)
    {
        var cmd = "segments("+x1+", "+y1+", "+x2+", "+y2+")";
        this.stat_connector.EvaluateNoReturn(cmd);        
    }
}

jsR_project.prototype.Evaluate = function(command)
{
    if (this.IsOpen)
    {
        return this.stat_connector.Evaluate(command);        
    }
}

jsR_project.prototype.draw_x_axis = function()
{
    if (this.IsOpen)
    {
        this.stat_connector.EvaluateNoReturn("abline(h=0)");
    }
}

jsR_project.prototype.draw_y_axis = function()
{
    if (this.IsOpen)
    {
        this.stat_connector.EvaluateNoReturn("abline(v=0)");
    }
}

jsR_project.prototype.plot_line_value = function(value)
{
    if (this.IsOpen && value != undefined)
    {
        var cmd = "abline(v="+value+")";
        this.stat_connector.EvaluateNoReturn(cmd);
    }
}
//------------------------------------------------------------------------------
// check R
//------------------------------------------------------------------------------
function check_r()
{
var r = new jsR_project();
var LEN    = 10000;
var T = 0.5;

if (r.Open())
{
    write("Opening R...\n");
    r.NewFigure("Test R");
    r.HoldOn = true;
    r.Axis2D(0, 1.0, -1, 1, "Wave", "Time (s)", "Amplitude");

    var omega = 2.0*Math.PI/T;
    var i;
/*    
    var amp = new Array(LEN);
    var time = new Array(LEN);
    for(i = 0; i < LEN; i++)
    {
        var t = i/LEN;
        amp[i] =  Math.cos(omega*t);
        time[i] = t;
    }
*/

    var amp = Float64Vector(LEN);
    var time = Float64Vector(LEN);
    for(i = 0; i < LEN; i++)
    {
        var t = i/LEN;
        var a = Math.cos(omega*t);

        time.set(i, t);
        amp.set(i, a);
    }
    r.Plot2D(time, amp);
}
else
{
    write("Unable to start R\n");
}
}