//--------------------------------------------------------------------------
//  Objet marker
//
//  Gilles Dietrich, 2015
//--------------------------------------------------------------------------

require("matrix33")

function jsMarker(name_or_id)
{
    this.name = "";                             // nom du marqueur
    this.id = -1;                               // numro [0..num-1]
    this.X = undefined;;                        // donnes axe X
    this.Y = undefined;;                        // donnes axe Y
    this.Z = undefined;;                        // donnes axe 7
    this.isOk = false;                          // flag
    
    var max_id = Mi.GetMarkerCount();
    if (typeof(name_or_id) == "number")         //creation avec un numro [0..N-1]
    {
        if (name_or_id >=0 && name_or_id < max_id)
        {
            this.id = name_or_id;
            this.name = Mi.GetMarkerName(this.id);
        }
    }
    else if (typeof(name_or_id) == "string")    // creation avec le nom du marqueur
    {
        this.name = name_or_id;
        this.id = Mi.GetMarkerNum(this.name);
    }

    this.Load();                                    // chargement des donnes        
}

jsMarker.prototype.SetElement = function(dim, frame, value)
{
    var ret = false;
    if (this.isOk && typeof(frame) == "number" && typeof(dim) == "number")
    {
        var len = this.X.length;
        if (dim >=0 && dim < 3 && frame >=0 && frame < len)
        {
            ret = true;
            if (dim == 0) this.X[frame] = value;
            else if (dim == 1) this.Y[frame] = value;
            else this.Z[frame] = value;
        }
    }
    return ret;
}

jsMarker.prototype.GetElement = function(dim, frame)
{
    var ret = 0;
    if (this.isOk && typeof(frame) == "number" && typeof(dim) == "number")
    {
        var len = this.X.length;
        if (dim >=0 && dim < 3 && frame >=0 && frame < len)
        {
            if (dim == 0) ret = this.X[frame];
            else if (dim == 1) ret = this.Y[frame];
            else ret = this.Z[frame];
        }
    }
    return ret;
}

jsMarker.prototype.GetVector = function(frame)
{
    var ret = undefined;
    if (this.isOk && typeof(frame) == "number")
    {
        if (frame >=0 && frame < this.X.length)
        {
			ret = new jsVector3(this.name+" ["+frame.toString()+"]");
            ret.Data[0] = this.X[frame];
            ret.Data[1] = this.Y[frame];
            ret.Data[2] = this.Z[frame];
        }
    }
    return ret;
}

jsMarker.prototype.SetVector = function(frame, v)
{
    var ret = false;
    if (this.isOk && typeof(frame) == "number")
    {
        if (frame >=0 && frame < this.X.length && typeof(v) == 'object')
        {
            ret = true;
            this.X[frame]= v.Data[0];
            this.Y[frame]= v.Data[1];
            this.Z[frame]= v.Data[2];
        }
    }
    return ret;
}

jsMarker.prototype.GetLastValidFrame = function()
{
    var frame = undefined;
    if (this.isOk)
    {
        var len = this.X.length;
        var i;
        for(i = len-1; i >= 0; i--)
        {
            if (this.X[i] != 0 && this.Y[i] != 0 && this.Z[i] != 0) return i;
        }
        return 0;
    }
    return frame;
}

jsMarker.prototype.GetFirstValidFrame = function()
{
    var frame = undefined;
    if (this.isOk)
    {
        var len = this.X.length;
        var i;
        for(i = 0; i < len; i++)
        {
            if (this.X[i] != 0 && this.Y[i] != 0 && this.Z[i] != 0) return i;
        }
        return len-1;
    }
    return frame;
}

jsMarker.prototype.Scale = function(scale)
{
    var ok = false;
    if (this.isOk && typeof(scale) == "number")
    {
        var i, len;
        len = this.X.length;
        for(i = 0; i < len; i++)
            this.X[i]*=scale
        len = this.Y.length;
        for(i = 0; i < len; i++)
            this.Y[i]*=scale
        len = this.Z.length;
        for(i = 0; i < len; i++)
            this.Z[i]*=scale
        ok = true;
    }
    return ok;
}

jsMarker.prototype.Save = function()
{
    var ok = false;
    if (this.isOk)
    {
        Mi.PutKineDataBuffer(this.id, 0, this.X);
        Mi.PutKineDataBuffer(this.id, 1, this.Y);
        Mi.PutKineDataBuffer(this.id, 2, this.Z);
        ok = true;
    }
    return ok
}

jsMarker.prototype.Copy = function(name)
{
    var ok = false;
    var new_id = -1;
    var max_id = Mi.GetMarkerCount();
    if (typeof(name) == "string")
        new_id = Mi.GetMarkerNum(name);
    else if (typeof(name) == "number")
        new_id = name;
    if (new_id >=0 && new_id < max_id)
    {
        var old_id = this.id;
        this.id = new_id;
        ok = this.Load();
        this.id = old_id;
    }
    return ok;
}

jsMarker.prototype.CreateAngle = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewAngle(name);
    }
    return ok;
}

jsMarker.prototype.CreateScalar = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewScalar(name);
    }
    return ok;
}

jsMarker.prototype.CreateForce = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewForce(name);
    }
    return ok;
}

jsMarker.prototype.CreateMoment = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewMoment(name);
    }
    return ok;
}

jsMarker.prototype.CreatePower = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewPower(name);
    }
    return ok;
}

jsMarker.prototype.CreateReaction = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewReaction(name);
    }
    return ok;
}

jsMarker.prototype.Create = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetMarkerNum(name);
        if (new_id < 0)
        {
            Mi.NewMarker(name);
            new_id = Mi.GetMarkerNum(name);
            if (new_id >= 0)
            {
                this.id = new_id;
                this.name = name;
                ok = this.Load();
            }
        }
    }
    return ok;
}

jsMarker.prototype.SaveAs = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetMarkerNum(name);
        if (new_id < 0) ok = this.Create(name);
        else
        {
            this.id = new_id;
            this.name = name;
            ok = this.Save();
        }
    }
    return ok;
}

jsMarker.prototype.Load = function()
{
    var max_id = Mi.GetMarkerCount();
    this.isOk = false;
    if (this.id >= 0 && this.id < max_id)
    {
        var frames = Mi.GetFrameCount();
        this.X = Mi.GetKineDataBuffer(this.id, 0);
        this.Y = Mi.GetKineDataBuffer(this.id, 1);
        this.Z = Mi.GetKineDataBuffer(this.id, 2);
        if (this.X.length == frames && this.Y.length == frames && this.Z.length == frames)
            this.isOk = true;
    }
    return this.isOk;
}
//----------------------------------------------------------------------------------
function jsMarkerMarkerExist(name)
{
    var ok = false;
    var id = -1;
    if (typeof(name) == "string")
        id = Mi.GetMarkerNum(name);
    else if (typeof(name) == "number")
        id = name;
    if (id >=0 && id < Mi.GetMarkerCount())
        ok = true;
    return ok;
}

//
//  Ajout de fonction Marker  l'objet Mi
//
Mi.MarkerExists = jsMarkerMarkerExist;
