//--------------------------------------------------------------------------
//  Objet marker : utilisation des vecteurs et non des Array
//  Utiliser marker.js pour la version Array
//  Gilles Dietrich, 2015
//--------------------------------------------------------------------------

require("matrix33")
require("vecmat64")

function jsMarker64(name_or_id)
{
    this.name = "";                             // nom du marqueur
    this.id = -1;                               // numro [0..num-1]
    this.X = undefined;                         // donnes axe X
    this.Y = undefined;                         // donnes axe Y
    this.Z = undefined;                         // donnes axe 7
    this.isOk = false;                          // flag
    this.rate = Mi.GetRate();
    this.frames = Mi.GetFrameCount();

    var max_id = Mi.GetMarkerCount();
    if (typeof(name_or_id) == "number")         //creation avec un numro [0..N-1]
    {
        if (name_or_id >=0 && name_or_id < max_id)
        {
            this.id = name_or_id;
            this.name = Mi.GetMarkerName(this.id);
        }
    }
    else if (typeof(name_or_id) == "string")    // creation avec le nom du marqueur
    {
        this.name = name_or_id;
        this.id = Mi.GetMarkerNum(this.name);
    }
    this.Load();                                    // chargement des donnes        
}

jsMarker64.prototype.size = function()
{
    if (this.isOk)
        return this.frames;
    return 0;
}

jsMarker64.prototype.CheckCreate = function(dim, frame, value)
{
    if (typeof(this.name) == "string" && this.id < 0)
        this.Create(this.name);
}

jsMarker64.prototype.SetElement = function(dim, frame, value)
{
    var ret = false;
    if (this.isOk && typeof(frame) == "number" && typeof(dim) == "number")
    {
        var len = this.X.size();
        if (dim >=0 && dim < 3 && frame >=0 && frame < len)
        {
            ret = true;
            if (dim == 0) this.X.set(frame, value);
            else if (dim == 1) this.Y.set(frame, value);
            else this.Z.set(frame, value);
        }
    }
    return ret;
}

jsMarker64.prototype.SetPoint = function(frame, p)
{
    if (this.isOk && typeof(frame) == "number")
    {
        var len = this.X.size();
        if (frame >=0 && frame < len)
        {
            this.SetElement(0, frame, p.GetElement(0));
            this.SetElement(1, frame, p.GetElement(1));
            this.SetElement(2, frame, p.GetElement(2));
        }
    }
}

jsMarker64.prototype.GetVector64 = function(dim)
{
    var ret = undefined;
    if (this.isOk && typeof(dim) == "number")
    {
        if (dim >=0 && dim < 3)
        {
            ret = new jsVector64(this.frames);
            var i;
            for(i = 0; i < this.frames; i++)
            {
                ret.SetElement(i, this.GetElement(i, dim));
            }
        }
    }
    return ret;
}

jsMarker64.prototype.GetPoint = function(frame)
{
    var p = new jsVector3(0);
    if (this.isOk && typeof(frame) == "number")
    {
        var len = this.X.size();
        if (frame >=0 && frame < len)
        {
            p.SetElement(0, this.GetElement(0, frame));
            p.SetElement(1, this.GetElement(1, frame));
            p.SetElement(2, this.GetElement(2, frame));
        }
    }
    return p;
}

jsMarker64.prototype.GetElement = function(dim, frame)
{
    var ret = 0;
    if (this.isOk && typeof(frame) == "number" && typeof(dim) == "number")
    {
        var len = this.X.size();
        if (dim == 1) len = this.Y.size();
        if (dim == 2) len = this.Z.size();
        if (dim >=0 && dim < 3 && frame >=0 && frame < len)
        {
            if (dim == 0) ret = this.X.get(frame);
            else if (dim == 1) ret = this.Y.get(frame);
            else ret = this.Z.get(frame);
        }
    }
    return ret;
}

jsMarker64.prototype.GetVector = function(frame)
{
    var ret = undefined;
    if (this.isOk && typeof(frame) == "number")
    {
        if (frame >=0 && frame < this.X.size())
        {
			ret = new jsVector3(this.name+" ["+frame.toString()+"]");
            ret.Data[0] = this.X.get(frame);
            ret.Data[1] = this.Y.get(frame);
            ret.Data[2] = this.Z.get(frame);
        }
    }
    return ret;
}

jsMarker64.prototype.SetVector = function(frame, v)
{
    var ret = false;
    if (this.isOk && typeof(frame) == "number")
    {
        if (frame >=0 && frame < this.X.size() && typeof(v) == 'object')
        {
            ret = true;
            this.X.set(frame, v.Data[0]);
            this.Y.set(frame, v.Data[1]);
            this.Z.set(frame, v.Data[2]);
        }
    }
    return ret;
}

jsMarker64.prototype.IsValidFrame = function(frame)
{
    if (this.isOk)
    {
        var len = this.X.size();
        if (frame >= 0 && frame < len)
        {
            if (this.X.get(frame) != 0 && this.Y.get(frame) != 0 && this.Z.get(frame) != 0) return true;
        }
    }
    return false;
}

jsMarker64.prototype.GetLastValidFrame = function()
{
    var frame = undefined;
    if (this.isOk)
    {
        var len = this.X.size();
        var i;
        for(i = len-1; i >= 0; i--)
        {
            if (this.X.get(i) != 0 && this.Y.get(i) != 0 && this.Z.get(i) != 0) return i;
        }
        return 0;
    }
    return frame;
}

jsMarker64.prototype.GetFirstValidFrame = function()
{
    var frame = undefined;
    if (this.isOk)
    {
        var len = this.X.size();
        var i;
        for(i = 0; i < len; i++)
        {
            if (this.X.get(i) != 0 && this.Y.get(i) != 0 && this.Z.get(i) != 0) return i;
        }
        return len-1;
    }
    return frame;
}

jsMarker64.prototype.Scale = function(scale)
{
    var ok = false;
    if (this.isOk && typeof(scale) == "number")
    {
        if (this.X)
            this.X.scale(scale);
        if (this.Y)
            this.Y.scale(scale);
        if (this.Z)
            this.Z.scale(scale);
        ok = true;
    }
    return ok;
}

jsMarker64.prototype.Save = function()
{
    var ok = false;
    if (this.isOk)
    {
        Mi.PutKineDataBuffer64(this.id, 0, this.X);
        Mi.PutKineDataBuffer64(this.id, 1, this.Y);
        Mi.PutKineDataBuffer64(this.id, 2, this.Z);
        ok = true;
    }
    return ok
}

jsMarker64.prototype.SaveAs = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetMarkerNum(name);
        if (new_id < 0) ok = this.Create(name);
        else
        {
            this.id = new_id;
            this.name = name;
            ok = this.Save();
        }
    }
    return ok;
}

jsMarker64.prototype.Load = function()
{
    var max_id = Mi.GetMarkerCount();
    this.isOk = false;
    if (this.id >= 0 && this.id < max_id)
    {
        this.X = Mi.GetKineDataBuffer64(this.id, 0);
        this.Y = Mi.GetKineDataBuffer64(this.id, 1);
        this.Z = Mi.GetKineDataBuffer64(this.id, 2);
        this.rate = Mi.GetRate();
        this.frames = Mi.GetFrameCount();
        this.isOk = true;
    }
    return this.isOk;
}

jsMarker64.prototype.Copy = function(name)
{
    var ok = false;
    var new_id = -1;
    var max_id = Mi.GetMarkerCount();
    if (typeof(name) == "string")
        new_id = Mi.GetMarkerNum(name);
    else if (typeof(name) == "number")
        new_id = name;
    if (new_id >=0 && new_id < max_id)
    {
        var old_id = this.id;
        this.id = new_id;
        ok = this.Load();
        this.id = old_id;
    }
    return ok;
}

jsMarker64.prototype.CreateAngle = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewAngle(name);
    }
    return ok;
}

jsMarker64.prototype.CreateScalar = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewScalar(name);
    }
    return ok;
}

jsMarker64.prototype.CreateForce = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewForce(name);
    }
    return ok;
}

jsMarker64.prototype.CreateMoment = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewMoment(name);
    }
    return ok;
}

jsMarker64.prototype.CreatePower = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewPower(name);
    }
    return ok;
}

jsMarker64.prototype.CreateReaction = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        ok = this.Create(name);
        if (ok)
            Mi.NewReaction(name);
    }
    return ok;
}

jsMarker64.prototype.Create = function(name)
{
    var ok = false;
    if (typeof(name) == "string")
    {
        var new_id = Mi.GetMarkerNum(name);
        if (new_id < 0)
        {
            Mi.NewMarker(name);
            new_id = Mi.GetMarkerNum(name);
            if (new_id >= 0)
            {
                this.id = new_id;
                this.name = name;
                ok = this.Load();
            }
        }
    }
    return ok;
}

//----------------------------------------------------------------------------------
function jsMarkerExist(name)
{
    var ok = false;
    var id = -1;
    if (typeof(name) == "string")
        id = Mi.GetMarkerNum(name);
    else if (typeof(name) == "number")
        id = name;
    if (id >=0 && id < Mi.GetMarkerCount())
        ok = true;
    return ok;
}

//
//  Ajout de fonction Marker  l'objet Mi
//
Mi.MarkerExists = jsMarkerExist;
