//---------------------------------------------------------------------
// Matrix33.js
// 3D vector and matrix
// Gilles Dietrich 2015
//---------------------------------------------------------------------
function jsVector3(n)
{
    this.Data = new Array(3);
    var i;
    this.type = "Vector3";

    this.name = "";
    var init_value = 0;
    if (typeof(n) == "string")
        this.name = n;
    else if (typeof(n) == "number")
        init_value = n;
    for(i = 0; i < 3; i++)
        this.Data[i] = init_value;
}


jsVector3.prototype.Init = function(v)
{
    var i;
    for(i = 0; i < 3; i++)
        this.Data[i] = v;
}

jsVector3.prototype.DistanceSquare = function(v)
{
    var ret = 0;
    if (typeof(v) == "object")
    {
        if (v.type == "Vector3")
        {
            var i;
            for(i = 0; i < 3; i++)
            {
                var c = this.Data[i] - v.Data[i];
                ret+=(c*c);
            }
        }
    }
    return ret;
}

jsVector3.prototype.Distance = function(v)
{
    var ret = this.DistanceSquare(v);
    return Math.sqrt(ret);
}

jsVector3.prototype.Norme = function()
{
    var n = Math.sqrt(this.Data[0]*this.Data[0]+ this.Data[1]*this.Data[1] + this.Data[2]*this.Data[2]);
    return n;
}

jsVector3.prototype.Normalize = function()
{
    var ret = undefined;
    var n = this.Norme();
    if (n > 0)
    {
        ret = new jsVector3();
        var i;
        for(i = 0; i < 3; i++)
            ret.Data[i] = this.Data[i] / n;
    }
    else ret = this.Copy();
    return ret;
}

jsVector3.prototype.CrossProduct = function(v)
{
	var ret = undefined;
	if (typeof(v) == 'object')
	{
        if (v.type == "Vector3")
		{
			ret = new jsVector3();
			ret.Data[0] = this.Data[1]*v.Data[2] - this.Data[2]*v.Data[1];
			ret.Data[1] = this.Data[2]*v.Data[0] - this.Data[0]*v.Data[2];
			ret.Data[2] = this.Data[0]*v.Data[1] - this.Data[1]*v.Data[0];
		}
	}
	return ret;
}

jsVector3.prototype.DotProduct = function(v)
{
	var ret = undefined;	
	if (typeof(v) == 'object')
	{
        if (v.type == "Vector3")
		{
			var i;
			ret = 0;
			for(i = 0; i < 3; i++)
			{
				ret = ret + this.Data[i] * v.Data[i];
			}
		}
	}
	return ret;
}

jsVector3.prototype.Print = function()
{
	var i;
	write(this.name+ " = [")
	for(i=0; i < 3; i++)
	{
		write(this.Data[i]);
		if (i < 2)
			write(" ,");
	}
	write("]\n");
}


jsVector3.prototype.Scale = function(scale)
{
    var ret = new jsVector3();
    var s = 1.0;
	if (typeof(scale) == 'number')
        s  = scale;
    var i;
    for(i = 0; i < 3; i++)
        ret.Data[i] = this.Data[i]*scale;
	return ret;
}

jsVector3.prototype.Add = function(v)
{
	var ret = undefined;	
	if (typeof(v) == 'object')
	{
        if (v.type == "Vector3")
		{
			ret = new jsVector3();
			var i;
			for(i = 0; i < 3; i++)
			{
				ret.Data[i] = this.Data[i] + v.Data[i];
			}
		}
	}
	return ret;
}

jsVector3.prototype.Sub = function(v)
{
	var ret = undefined;
	if (typeof(v) == 'object')
	{
        if (v.type == "Vector3")
		{
			ret = new jsVector3(0);
			var i;
			for(i = 0; i < 3; i++)
			{
				ret.Data[i] = this.Data[i] - v.Data[i];
			}
		}
	}
	return ret;
}

jsVector3.prototype.Copy = function(v)
{
	var ret = undefined;
	if (typeof(v) == 'object')
	{
        if (v.type == "Vector3")
		{
			var i;
			for(i = 0; i < 3; i++)
				this.Data[i] = v.Data[i];
		}
	}
    else
    {
        ret  = new jsVector3();
        var i;
        for(i = 0; i < 3; i++)
            ret.Data[i] = this.Data[i];
    }
	return ret;
}

jsVector3.prototype.GetElement = function(col)
{
    var v = undefined;
    if (typeof(col) == "number")
    {
        if (col >=0 && col < 3 )
            v = this.Data[col];
    }
    return v;
}

jsVector3.prototype.SetElement = function(col, value)
{
    if (typeof(col) == "number"  && typeof(value) == "number")
    {
        this.Data[col]= value;
    }
}
//--------------------------------------------------------------------
//  jsMatrix33
//--------------------------------------------------------------------
function jsMatrix33(n)
{
    var i, j;
    this.Data = new Array(3);
    for(i = 0; i < 3; i++)
        this.Data[i] = new Array(3);
    this.name = "";
    this.type = "Matrix33";
    var init_value = 0;
    if (typeof(n) == "string")
    {
        this.name = n;
    }
    else if (typeof(n) == "number")
    {
        init_value = n;
    }
    var i, j;
        for(i = 0; i < 3; i++)
            for(j = 0; j < 3; j++)
				this.Data[i][j] = init_value;
}

jsMatrix33.prototype.Determinant = function()
{
    var d;
    d =  this.Data[0][0]*(this.Data[1][1]*this.Data[2][2] - this.Data[2][1]*this.Data[1][2]) +
         this.Data[0][1]*(this.Data[1][0]*this.Data[2][2] - this.Data[2][0]*this.Data[1][2]) + 
         this.Data[0][2]*(this.Data[1][0]*this.Data[2][1] - this.Data[2][0]*this.Data[1][1]);
    return d;
}

jsMatrix33.prototype.Inverse = function()
{
    var i, j, k;
    var IMat = this.Copy();
    for(i=0; i < 3; i++)
    {
        var q = IMat.Data[i][i];
        if (Math.abs(q) <= 1e-16) return undefined;
//        if (Math.abs(q) <= Number.EPSILON) return undefined;
        IMat.Data[i][i] = 1.0;
        if (q != 1)
        {
            for(k = 0; k < 3; k++)
                IMat.Data[i][k] = IMat.Data[i][k]/q;
            
        }		// q <> 1
        for(j = 0; j < 3; j++)
        {
            if (i != j)
            {
                q = IMat.Data[j][i];
                IMat.Data[j][i] = 0.0;
                for(k =0; k < 3; k++)
                    IMat.Data[j][k] = IMat.Data[j][k]-q*IMat.Data[i][k];
            }
        }
    }
    return IMat;
}

jsMatrix33.prototype.Init = function(v)
{
    if (typeof(v) == "number")
    {
        var i, j;
        for(i = 0; i < 3; i++)
            for(j = 0; j < 3; j++)
				this.Data[i][j] = v;
    }
}

jsMatrix33.prototype.Copy = function(data)
{
    var ret = undefined;
	if (typeof(data) == 'object')
	{
        var i, j;
        for(i = 0; i < 3; i++)
            for(j = 0; j < 3; j++)
                this.Data[i][j] = data.Data[i][j];
    }
    else
    {
        ret = new jsMatrix33();
        var i, j;
        for(i = 0; i < 3; i++)
            for(j = 0; j< 3; j++)
                ret.Data[i][j] = this.Data[i][j];
    }
    return ret;
    
}

jsMatrix33.prototype.Multiply = function(m)
{
    var ret = undefined;
    if (typeof(m) == "number")
    {
        ret = new jsMatrix33();
        var i, j;
        for(i = 0; i < 3; i++)
            for(j = 0; j < 3; j++)
                ret.Data[i][j] = this.Data[i][j]*m;
    }
    else if (typeof(m) == "object")
    {
        if (m.type == "Vector3")
        {
            var i, j;
            ret = new jsVector3();
            for(i = 0; i < 3; i++)
            {
                var v = 0;
                for(j = 0; j < 3; j++)
                {
                    v = v + m.Data[j]*this.Data[j][i];
                }
                ret.Data[i] = v;
            }
            
        }
        else if (m.type == "Matrix33")
        {
            var i, j, k;
            ret = new jsMatrix33();
            for(i = 0; i < 3; i++)
            {
                for(j = 0; j < 3; j++)
                {
                    ret.Data[i][j] = 0;
                    for(k = 0; k< 3; k++)
                    {
                        ret.Data[i][j] = ret.Data[i][j] + this.Data[k][j]*m.Data[i][k];
                    }
                }
            }
        }
    }
    return ret;
}

jsMatrix33.prototype.Scale = function(scale)
{
    var s = 1.0;
    if (typeof(scale) == "number")
        s = scale;
    var ret = new jsMatrix33();
    var i, j;
    for(i = 0; i < 3; i++)
        for(j = 0; j < 3; j++)
            ret.Data[i][j] = this.Data[i][j]*s;
    return ret;
}

jsMatrix33.prototype.Transpose = function()
{
	var ret = new jsMatrix33();
    var i, j;
    for(i=0; i < 3; i++)
    {
        for(j = 0; j < 3; j++)
        {
            ret.Data[j][i] = this.Data[i][j];
        }
	}
	return ret;
}

jsMatrix33.prototype.MakeId = function()
{
	var col, row;
	for(col = 0; col < 3; col++)
	{	for(row = 0; row < 3; row++)              
		{
			if (col == row) this.Data[col][row] = 1.0;
			else this.Data[col][row] = 0.0;
		}
	}
    return this;
}

jsMatrix33.prototype.Print = function()
{
	var col, row;
	write(this.name+" = \n");
	for(row = 0; row < 3; row++)
	{	write("\t[");
		for(col = 0; col < 3; col++)              
		{
			write(this.Data[col][row]);
            if (col < 2)
                write("\t");
		}
		write("]\n");
	}
}

jsMatrix33.prototype.GetElement = function(col, row)
{
    var v = undefined;
    if (typeof(col) == "number" && typeof(row) == "number")
    {
        if (col >=0 && col < 3 && row >= 0 && row < 3)
            v = this.Data[col][row];
    }
    return v;
}

jsMatrix33.prototype.SetElement = function(col, row, value)
{
    if (typeof(col) == "number" && typeof(row) == "number" && typeof(value) == "number")
        this.Data[col][row]= value;
}

jsMatrix33.prototype.FindLargestEntry =function()
{
    var result=0.0;
    var i, j;
    for(i=0;i<3;i++)
    {
        for(j=0;j<3;j++)
        {
            var  entry =Math.abs(this.GetElement(i,j));
            result = Math.max(entry,result);
        }
    }
    return result;
}

jsMatrix33.prototype.FindEigenVectorAssociatedWithLargestEigenValue = function()
{
    var v = undefined;
    var  scale = this.FindLargestEntry();
    if (scale != 0)
    {
        var i;
        var mc = this.Scale(1.0/scale);
        mc = mc.Multiply(mc);
        mc = mc.Multiply(mc);
        mc = mc.Multiply(mc);
        var v = new jsVector3("EigenVector");
        for(i = 0; i < 3; i++)
            v.SetElement(i, 1);
        var lastV = v.Copy();
        for(i = 0; i < 100;i++)
        {
            v = mc.Multiply(v);
            v = v.Normalize();
            if(v.DistanceSquare(lastV) < 1e-16) break;
            lastV = v.Copy();
        }        
    }
    return v;
} 

jsMatrix33.prototype.FromVector = function(v1, v2, a1, a2)
{
    if (a1 != a2)
    {
        var a3 = 3-a1-a2;
        var a = v1.Normalize();
        var c = new jsVector3();
        var b = new jsVector3();
        if ( ((a1==0) && (a2==1)) || ((a1==1) && (a2==2)) || ((a1==2) && (a2==0)) )
        {
            var c0 = a.CrossProduct(v2);
            c = c0.Normalize();
            var n = c.Norme();
            if (n < 1e-16)
            {
//                this.Init(0);
                this.MakeId();
                return;
            }
            b = c.CrossProduct(a);
        }
        else
        {
            var c0 = a.CrossProduct(v2);
            c = c0.Normalize(c);
            var n = c.Norme();
            if (n < 1e-16)
            {
//                this.Init(0);
                 this.MakeId();
                return;
            }
            b = a.CrossProduct(c);
        }
        var i;
        for(i = 0; i < 3; i++)
        {
            this.SetElement(a1, i, a.GetElement(i));
            this.SetElement(a2, i, b.GetElement(i));
            this.SetElement(a3, i, c.GetElement(i));
        }
    }
    else this.MakeId();
}

jsMatrix33.prototype.FromFrame = function(p1, p2, p3, a1, a2)
{
    var v1 = p2.Sub(p1);
    var v2 = p3.Sub(p1);
    return this.FromVector(v1, v2, a1, a2);    
}
//------------------------------------------------------------------
//  non object function
//------------------------------------------------------------------
function jsToLocal(p1, p2, p3, a1, a2, point, origin)
{
    var p = point.Sub(origin);
    var o = new jsMatrix33(0);
    o.FromFrame(p1, p2, p3, a1, a2);
    if (typeof(o) != "undefined")
    {
        var Iv = o.Inverse();
        p  = Iv.Multiply(p);
    }
    return p;
    /*
    var a = new Array(3);
    var i;
    for(i = 0; i < 3; i++)
        a[i] = p.GetElement(i);
    return a;
    */
}

function jsToMain(p1, p2, p3, a1, a2, point, origin)
{
    var o = new jsMatrix33(0);
    var p = new jsVector3(0);
    o.FromFrame(p1, p2, p3, a1, a2);
    if (typeof(o) != "undefined")
    {
        p = o.Multiply(point);
        p = p.Add(origin);
    }
    return p;
    /*
    var a = new Array(3);
    var i;
    for(i = 0; i < 3; i++)
        a[i] = p.GetElement(i);
    return a;
    */
}

function jsFindLLSQPlane(points, count, verbose)
// points : array[3][count] calibration point
// verbose : boolean flag 
{
    var ret;
    if (verbose)
        print("Find plane from ",+count+" points");
    if (count > 0)
    {
        ret = new Array(3); // [0] : Normal, [1] : Center [2] : D
        var sum = new Array(3);
        var i, j;
        for(i = 0; i < 3; i++)
            sum[i] = 0.0;
        for(j = 0; j < count; j++)
        {
            for(i = 0; i < 3; i++)
                sum[i] = sum[i]+points[i][j];
        }
        var center = new Array(3);
        var normal = new Array(3);
        for(i = 0; i < 3; i++)
        {
            normal[i] = 0.0;
            center[i]= sum[i]/count;
        }
        var sumXX =0.0,sumXY =0.0, sumXZ =0.0;
        var sumYY =0.0,sumYZ =0.0;
        var sumZZ =0.0;

        for(j = 0; j < count; j++)
        {
            var diffX =points[0][j]-center[0];
            var diffY =points[1][j]-center[1];
            var diffZ =points[2][j]-center[2];
            
            sumXX =sumXX + diffX*diffX;
            sumXY =sumXY + diffX*diffY;
            sumXZ =sumXZ + diffX*diffZ;
            sumYY =sumYY + diffY*diffY;
            sumYZ =sumYZ + diffY*diffZ;
            sumZZ =sumZZ + diffZ*diffZ;
        }
        var m = new jsMatrix33("m");
        m.SetElement(0, 0, sumXX); m.SetElement(1, 0, sumXY); m.SetElement(2, 0, sumXZ); 
        m.SetElement(0, 1, sumXY); m.SetElement(1, 1, sumYY); m.SetElement(2, 1, sumYZ); 
        m.SetElement(0, 2, sumXZ); m.SetElement(1, 2, sumYZ); m.SetElement(2, 2, sumZZ); 
        var det = m.Determinant();
        if (det != 0)
        {
            var mi = m.Inverse();
            var n = mi.FindEigenVectorAssociatedWithLargestEigenValue();
            for(i = 0; i < 3; i++)
                normal[i] = n.GetElement(i);
            var d = -(center[0]*normal[0] + center[1]*normal[1] + center[2]*normal[2]);
            ret[0] = center;
            ret[1] = normal;
            ret[2] = d;
        }
    }
    return ret;
}
//------------------------------------------------------------------------------
function test_Matrix33()
{

	var mat = new jsMatrix33("mat");
    mat.Init(0);
    mat.MakeId();
    mat.Print();

	var mat2 = mat.Copy();
    mat2.name = "mat2";
    mat2 = mat2.Multiply(0.5);
    mat2.Transpose();
    mat2.Print();
    
    var v = mat2.FindEigenVectorAssociatedWithLargestEigenValue();
    v.name = "Eigen Vector";
    v.Print();
    
    v = new jsVector3(3);
    var i;
    for(i = 0; i < 3; i++)
        v.SetElement(i, i);
    v.Scale(2);
    v.Print();

    var v2 = mat2.Multiply(v);
    v2.Print();
    
    var m3 = mat2.Multiply(mat);
    var m4 = m3.Inverse();
    m3.Print();
    m4.Print();
    var m5 = m4.Multiply(m3);
    var s = m5.Determinant();
    m5.Print();
    

    var v1 = new jsVector3();
    v1.SetElement(0, 0);
    v1.SetElement(1, 1);
    v1.SetElement(2, 0);
    v1.Print();
    
    var v2 = new jsVector3(0);
    v2.SetElement(0, 1);
    v2.SetElement(1, 0);
    v2.SetElement(2, 0);
    v2.Print();

    var m5 = new jsMatrix33("m5");
    m5.Print();
    m5.FromVector(v1, v2, 2, 0);
    m5.Print();

    m5 = new jsMatrix33("m5");

    var p1 = new jsVector3("p1");
    p1.SetElement(0, 1);
    p1.SetElement(1, 0);
    p1.SetElement(2, 0);
    p1.Print();

    var p2 = new jsVector3("p2");
    p2.SetElement(0, 0);
    p2.SetElement(1, 1);
    p2.SetElement(2, 0);
    p2.Print();

    var p3 = new jsVector3("p3");
    p3.SetElement(0, 1);
    p3.SetElement(1, 1);
    p3.SetElement(2, 1);

    m5.FromFrame(p1, p2, p3, 0, 2);
    m5.name = "Orient";
    m5.Print();
}

//test_Matrix33();
