//--------------------------------------------------------------------
//	Calcul de matrices et vecteurs simples
//
//	vecmat.js
//	Gilles Dietrich
//--------------------------------------------------------------------

//--------------------------------------------------------------------
//	Object vecteur
//--------------------------------------------------------------------
function jsVector(rows)
// objet vecteur
{
	this.v_name = "";						// nom symbolique
	this.v_rows = rows;						// nombre de ligne (identique  this.v_Data.length)

	this.Print = _PrintVec;					// affichage lisible : v1.Print();
	this.Init = _InitVec;					// initialisation avec une valeur : v1.Init(0.0);
	this.Norm = _NormVec;					// retourne la norme du vecteur	: var n = v.Norm();
	this.Max = _MaxVec;						// retourne le max : var max = v.Max();
	this.Min = _MinVec;						// retourne le min : var min = v.Min();
	this.Mean = _MeanVec;					// retourne la moyenne : var mean = v.Mean();
	this.Sd = _SdVec;						// retourne l'cart-type : var sd = v.Sd();

	this.Normalize = _NormalizeVec;			// normalise le vecteur v1.Normalize();
	this.DotProduct = _DotProductVec;		// produit scalaire var d = v1.DotProduct(v2);
	this.Distance = _DistanceVec;			// distance entre deux vecteurs var d = v1.Distance(v2);
	this.Copy = _CopyVec;					// copie d'un vecteur v1.Copy(v2);
	this.Add = _AddVec;						// addition de deux vecteurs : var v3 = v1.Add(v2);
	this.Sub = _SubVec;						// diffrence de 2 vecteurs : var v3 = v1.Sub(v2);
	this.Scale = _ScaleVec;
	// Initialisation du vecteur
	if (rows > 0)
	{
		this.v_Data = new Array(rows);		// donne (normalement des nombres rels)
		var i;
		for(i = 0; i < rows; i++)
			this.v_Data[i] = 0.0;
	}
}

function _ScaleVec(scale)
{
    var ret = false;
    if (typeof(scale) == "number" && this.v_rows > 0)
    {
        ret= 0;
        var i;
        for(i = 0; i < this.v_rows; i++)
            this.v_Data[i]*=scale;
    }
    return ret;
}

function _AddVec(v)
{
	var ret;	
	if (typeof(v) == 'object')
	{
		if (v.v_rows == this.v_rows && v.v_rows > 0)
		{
			ret = new jsVector(v.v_rows);
			var i;
			for(i = 0; i < v.v_rows; i++)
			{
				ret.v_Data[i] = this.v_Data[i] + v.v_Data[i];
			}
		}
	}
	return ret;
}

function _SubVec(v)
{
	var ret;
	if (typeof(v) == 'object')
	{
		if (v.v_rows == this.v_rows && v.v_rows > 0)
		{
			ret = new jsVector(v.v_rows);
			var i;
			for(i = 0; i < v.v_rows; i++)
			{
				ret.v_Data[i] = this.v_Data[i] - v.v_Data[i];
			}
		}
	}
	return ret;
}

function _CopyVec(v)
{
	if (typeof(v) == 'object')
	{
		if (v.v_rows > 0)
		{
			var i;
			this.v_Data = new Array(v.v_rows);
			for(i = 0; i < v.v_rows; i++)
				this.v_Data[i] = v.v_Data[i];
			this.v_rows = v.v_rows;
		}
	}
}

function _DistanceVec(v)
{
	var ret;
	if (typeof(v) == 'object')
	{
		if (this.v_rows == v.v_rows)
		{
			var tmp = this.Sub(v);
			ret = tmp.Norm();
		}
	}
	return ret;
}

function _CrossProductVec3(v)
{
	var ret;
	if (typeof(v) == 'object')
	{
		if (this.v_rows == v.v_rows && this.v_rows == 3)
		{
			ret = new jsVector(v.v_rows);
			ret.v_Data[0] = this.v_Data[1]*v.v_Data[2] - this.v_Data[2]*v.v_Data[1];
			ret.v_Data[1] = this.v_Data[2]*v.v_Data[0] - this.v_Data[0]*v.v_Data[2];
			ret.v_Data[2] = this.v_Data[0]*v.v_Data[1] - this.v_Data[1]*v.v_Data[0];
		}
	}
	return ret;
}

function _DotProductVec(v)
{
	var ret;	
	if (typeof(v) == 'object')
	{
		if (this.v_rows == v.v_rows)
		{
			var i;
			ret = 0;
			for(i = 0; i < v.v_rows; i++)
			{
				ret = ret + this.v_Data[i] * v.v_Data[i];
			}
		}
	}
	return ret;
}

function _MeanVec()
{
	var mean = 0;
	var i;
	for(i  = 0; i < this.v_rows; i++)
		mean = this.v_Data[i] + mean;
	mean = mean/(this.v_rows);
	return mean;
}

function _SdVec()
{
	var i;
	var s = 0;
	var s2 = 0;
	var sd = 0;
	var n = this.v_rows;
	for(i = 0; i < n; i++)
	{
		s  = s + this.v_Data[i];
		s2 = s2 + this.v_Data[i]*this.v_Data[i];
	}
	sd = (((n*s2)-(s*s))/(n*(n-1)));
	if (sd > 0)
		sd = Math.sqrt(sd);
	else
		sd = 0;
	return sd;
}

function _MaxVec()
{
	var max  = this.v_Data[0];
	var i;
	for(i  = 0; i < this.v_rows; i++)
		if (this.v_Data[i] > max)
			max = this.v_Data[i];
	return max;
}

function _MinVec()
{
	var min  = this.v_Data[0];
	var i;
	for(i  = 0; i < this.v_rows; i++)
		if (this.v_Data[i] < min)
			min = this.v_Data[i];
	return min;
}

function _NormVec()
{
	var i;
	var norm = 0.0;
	for(i = 0; i < this.v_rows; i++)
	{
		var c = this.v_Data[i]*this.v_Data[i];
		norm = norm + c;
	}
	norm = Math.sqrt(norm);
	return norm;
}	

function _NormalizeVec()
{
	var i;
	var norm = this.Norm();
	if (norm > 0)
	{
		for(i = 0; i < this.v_rows; i++)
		{
			this.v_Data[i] = this.v_Data[i] / norm;
		}
	}
}

function _InitVec(value)
{
	var i;
	for(i = 0; i < this.v_rows; i++)
		this.v_Data[i]=value;
}

function _PrintVec()
{
	var i;
	write(this.name+ " = [")
	for(i=0; i < this.v_rows; i++)
	{
		write(this.v_Data[i]);
		if (i < this.v_rows-1)
			write(" ,");
	}
	write("]\n");
}

//--------------------------------------------------------------------
//	Object Matrice
//--------------------------------------------------------------------
function jsMatrix(cols, rows)
{
	this.m_cols = cols;
	this.m_rows = rows
	if (cols > 0)
	{
		this.m_Data = new Array(cols);
		var i;
		if (rows > 0)
			for(i=0; i < cols; i++)
			{
				this.m_Data[i] = new Array(rows);
			}
	}
	this.name = "";									// proprit 'nom'

	this.Init	= _InitMat;							// initialisation de la matrice avec une valeur unique : m.Init(0.0);
	this.MakeId = _MakeMatId;						// matrice identit : v.MakeId();
	this.Print = _PrintMat;							// affichage des donnes de la matrice : m.Print();
	this.Transpose = _TransposeMat					// transpose la matrice : var mt = m.Transpose();
	this.Copy = _CopyMat;							// copie des donnes d'une matrice : m.Copy(m1);
}

function _InitMat(v)
{
	if (this.m_rows > 0 && this.m_cols > 0)
	{
		var i, j;
		for(i = 0; i < this.m_cols; i++)
			for(j = 0; j < this.m_rows; j++)
				this.m_Data[i][j] = v;
	}
}

function _CopyMat(data)
{
	if (typeof(data) == 'object')
	{
		if (data.m_rows > 0 && data.m_cols > 0)
		{
			var i, j;
			var tmp = new Array(data.m_cols);
			for(i = 0; i < data.m_cols; i++)
				tmp[i] = new Array(data.m_rows);
			this.m_rows = data.m_rows;
			this.m_cols = data.m_cols;
			for(i = 0; i < data.m_cols; i++)
				for(j = 0; j < data.m_rows; j++)
					tmp[i][j] = data.m_Data[i][j];
			this.m_Data = tmp;
		}
	}
}

function _TransposeMat()
{
	var ret;
	if (this.m_rows > 0 && this.m_cols > 0)
	{
		var new_cols = this.m_rows;
		var new_rows = this.m_cols;
		ret = new jsMatrix(new_cols, new_rows);
		var i, j;
		for(i=0; i < this.m_cols; i++)
		{
			for(j = 0; j < this.m_rows; j++)
			{
				ret.m_Data[j][i] = this.m_Data[i][j];
			}
		}
	}
	return ret;
}

function _MakeMatId()
{
	var col, row;
	for(col = 0; col < this.m_cols; col++)
	{	for(row = 0; row < this.m_rows; row++)              
		{
			if (col == row) this.m_Data[col][row] = 1.0;
			else this.m_Data[col][row] = 0.0;
		}
	}
}
function _PrintMat()
{
	var col, row;
	write(this.name+" = \n");
	for(row = 0; row < this.m_rows; row++)
	{	write("\t[");
		for(col = 0; col < this.m_cols; col++)              
		{
			write(this.m_Data[col][row]);
            if (col < this.m_cols-1)
                write("\t");
		}
		write("]\n");
	}
}
//-------------------------------------------------------------------------
//	Exemples d'application de la bibliothque
//-------------------------------------------------------------------------
function test_vecmat()
{
	var Vec = new jsVector(3);
	Vec.name = "vec";
	Vec.Init(Math.PI);
	Vec.Normalize();
	Vec.Print();
	var min = Vec.Min();
	var max = Vec.Max();
	var mean = Vec.Mean();
	var sd = Vec.Sd();
	print("Stat = "+ min+' , '+max+' , '+mean+' , '+sd);

	var v2 = new jsVector(3);				//
	v2.v_Data[0] = 0.0;						// assignation directe de valeurs
	v2.v_Data[1] = 1.0;
	v2.v_Data[2] = 0.0;

	var d  = Vec.DotProduct(v2);			// renvoie un scalaire
	var v3 = Vec.CrossProduct3(v2);			// renvoie un vecteur
	print("Dot = "+d);
	v3.name = "CrossProduct";
	v3.Print();

	var v4 = v2.Add(v3);					// renvoie un vecteur v4 = v2 + v3
	var v5 = v2.Sub(v3);					// renvoie un vecteur v5 = v2 - v3
	v4.name = "Add";
	v4.Print();

	d = v2.Distance(v3);					// renvoie un scalaire
	print("Distance v2 v3= "+d);

	var Mat = new jsMatrix(4, 3);			// cration d'une matrice 3x3
	Mat.name = "mat";
	Mat.MakeId();
	var Mt = Mat.Transpose();				// renvoie une matrice
	Mt.name = "Transpose";
	Mt.Print();

	var t = new jsMatrix(3, 3);
	t.Init(Math.sqrt(2.0));
	Mat.Copy(t);							// copie dans Mat les donnes de t
	Mat.name = "T";
	Mat.Print();
}

//test_vecmat();
