//--------------------------------------------------------------------
//	Calcul de vecteurs simples
//
//	vecmat.js
//	Gilles Dietrich
//--------------------------------------------------------------------

//--------------------------------------------------------------------
//	Object vecteur
//--------------------------------------------------------------------
function jsVector64(rows)
// objet vecteur
{
	this.name = "jsVector64";						        // nom symbolique
    this.Data = undefined;
    this.Invalid = undefined;
	// Initialisation du vecteur
	if (rows > 0)
        this.SetSize(rows);
}

jsVector64.prototype.SetSize = function(n)
{
    delete this.Data;
    this.Data = undefined;
    if (n > 0)
    {
        this.Data =  new Float64Vector (n);		// donne (normalement des nombres rels)
		var i;
		for(i = 0; i < n; i++)
			this.Data.set(i, 0.0);
    }
}

jsVector64.prototype.Size = function()
{
    var size = 0;
    if (this.Data)
        size = this.Data.size();
    return size;
}

jsVector64.prototype.size = function()
{
    return this.Size();
}

jsVector64.prototype.Assign = function(vector)
// vector could be jsVector64 or Float64Vector
{
	if (typeof(vector) == 'object')
    {
        var n = vector.size();
        this.SetSize(n);
        if (n > 0)
        {
            var i;
            for(i = 0; i < n; i++)
                this.set(i, vector.get(i));
        }
    }
}

jsVector64.prototype.CreateCopy = function()
{
    var c = new jsVector64();
    c.Assign(this);
    return c;
}

jsVector64.prototype.SetElement = function(idx, value)
{
    if (typeof(idx) == "number" && typeof(value) == "number")
    {
        if (idx >=0 && idx < this.Size())
            this.Data.set(idx, value);
    }
}

jsVector64.prototype.set = function(idx, value)
{
    this.SetElement(idx, value);
}

jsVector64.prototype.GetElement = function(idx)
{
    if (typeof(idx) == "number")
    {
        if (idx >=0 && idx < this.Size())
            return this.Data.get(idx);
    }
    return undefined;
}

jsVector64.prototype.get = function(idx)
{
    return this.GetElement(idx);
}

jsVector64.prototype.Scale = function(scale)
{
    if (typeof(scale) == "number")
        this.Data.scale(scale);
}

jsVector64.prototype.Add = function(vector)
{
	if (typeof(vector) == 'object')
    {
        var size = this.size();
        var s1 = vector.size();
        if (s1 < size) size = s1;
        var i;
        for(i = 0; i < size; i++)
        {
            var value = this.get(i) + vector.get(i);
            this.set(i, value);
        }
    }
 }

jsVector64.prototype.Sub = function(vector)
{
	if (typeof(vector) == 'object')
    {
        var size = this.size();
        var s1 = vector.size();
        if (s1 < size) size = s1;
        var i;
        for(i = 0; i < size; i++)
        {
            var value = this.get(i) - vector.get(i);
            this.set(i, value);
        }
    }
}

jsVector64.prototype.Norm = function()
{
    var n = 0;
    var size = this.size();
    var i;
    for(i = 0; i < size; i++)
    {
        var v = this.get(i)*this.get(i);
        n+=v;
    }
    if (n > 0) n = Math.sqrt(n);
    return n;
}

jsVector64.prototype.Normalize = function()
{
    var n = this.Norm();
    if (n > 0)
        this.Scale(1.0/n);
}

jsVector64.prototype.Distance = function(vector)
{
    var d = 0;
	if (typeof(vector) == 'object')
    {
        var tmp = this.CreateCopy();
        tmp.Sub(vector);
        d = tmp.Norm();
    }
    return d;
}

jsVector64.prototype.Print = function()
{
    var size = this.size();
    var i;
	for(i=0; i < size; i++)
	{
		write(this.get(i));
		if (i < size-1)
			write(" ,");
	}
	write("]\n");
    
}

jsVector64.prototype.Min = function()
{
    var size = this.size();
    var i;
    var m = undefined;
    if (size > 0)
    {
        m = this.get(0);
        for(i = 1; i < size; i++)
        {
            var v = this.get(i);
            if (v < m) m = v;
        }
    }
    return m;
}

jsVector64.prototype.MinIndex = function()
{
    var size = this.size();
    var i;
    var m = undefined;
    var idx = undefined;
    if (size > 0)
    {
        m = this.get(0);
        for(i = 1; i < size; i++)
        {
            var v = this.get(i);
            if (v < m) 
            {
                m = v;
                idx = i;
            }
        }
    }
    return idx;
}

jsVector64.prototype.Max = function()
{
    var size = this.size();
    var i;
    var m = undefined;
    if (size > 0)
    {
        m = this.get(0);
        for(i = 1; i < size; i++)
        {
            var v = this.get(i);
            if (v > m) m = v;
        }
    }
    return m;
}

jsVector64.prototype.MaxIndex = function()
{
    var size = this.size();
    var i;
    var m = undefined;
    var idx = undefined;
    if (size > 0)
    {
        m = this.get(0);
        for(i = 1; i < size; i++)
        {
            var v = this.get(i);
            if (v > m) 
            {
                m = v;
                idx = i;
            }
        }
    }
    return idx;
}

jsVector64.prototype.Energy = function()
{
    var size = this.size();
    var i;
    var e = 0;
    if (size > 0)
    {
        for(i = 0; i < size; i++)
            e+= this.get(i) * this.get(i);
    }
    return e;
}

jsVector64.prototype.Power = function()
{
    var p = this.Energy();
    var size = this.size();
    if (p > 0 && size > 0)
        p/=size;
    return p;
}

jsVector64.prototype.Rms = function()
{
    var rms = undefined;
    var size = this.size();
    if (size > 0)
    {
        var m = this.Mean();
        var i;
        rms = 0;
        for(i =0; i < size; i++)
            rms = rms + (m-this.get(i))*(m-this.get(i));
        rms = Math.sqrt(rms);
    }
    return rms;
}

jsVector64.prototype.Cv = function()
{
    var cv = undefined;
    var size = this.size();
    if (size > 0)
    {
        var m = this.Mean();
        var sd = this.StdDev();
        if (m != 0)
            cv = sd/m;
    }
    return cv;
}

jsVector64.prototype.Mean = function()
{
    var m = 0;
    var size = this.size();
    if (size > 0)
    {
        var i;
        for(i = 0; i < size; i++)
            m+=this.get(i);
        m/=size;
    }
    return m;
}

jsVector64.prototype.StdDev = function()
{
    var s = 0;
    var s2 = 0;
    var size = this.size();
    if (size > 0)
    {
        var i;
        for(i = 0; i < size; i++)
        {
            s+=this.get(i);
            s2+=(this.get(i)*this.get(i));
        }
        s = Math.sqrt(((size*s2)-(s*s))/(size*(size-1)));
    }
    return s;
}

jsVector64.prototype.Centile = function(cent)
{
    var c = undefined;
    var size = this.size();
    if (size > 0 && cent > 0 && cent < 100)
    {
        var tmp = new Float64Vector(size);
        var i;
        for(i = 0; i < size; i++)
            tmp.set(i, this.get(i));
        tmp.sort();
        var p = cent / 100.0;
        p*=size;
        var pc = Math.round(p);
        c = tmp.get(pc);
    }
    return c;
}
//-----------------------------------------------------------------------
//  test
//-----------------------------------------------------------------------
function test()
{
var Size = 1000;
var test = new jsVector64(Size);
var i;
for(i = 0; i < Size; i++)
    test.set(i, i);
var min = test.Min();
var max = test.Max();
var minidx = test.MinIndex();
var maxiex = test.MaxIndex();
var power = test.Power();
var sd = test.StdDev();
var mean = test.Mean();
var med = test.Centile(50);
var rms = test.Rms();

print("Min = "+min+"   Max = "+max);

//test.Print();

test.Scale(0.01);
var c = test.get(10);
print("c = "+c);
test.Normalize();

var ndata = new Float64Vector(1000);
test.Assign(ndata);

var t2 =  new jsVector64();
t2.Assign(test);
t2.Add(test);
t2.Sub(test);
var d = t2.Distance(test);
print("Distance = "+d);

}

//test();