//
//  file.c
// 
//  auteur : Gilles Dietrich, 2014

#include <miAPI.h>
#include <string.h>
#include "file.h"

int fiGetLastPos(char* str, char c)
{
	if (str != NULL)
	{
		int len = strlen(str);
		for(int i = len-1; i >= 0; i--)
		{
			if (str[i] == c) return i;
		}
	}
    return -1;
}

int fiGetFirstPos(char* str, char c)
{
	if (str != NULL)
	{
		int len = strlen(str);
		for(int i = 0; i < len; i++)
		{
			if (str[i] == c) return i;
		}
	}
    return -1;
}

int fiChangeExt(char* in_name, char* ext, char* out_name)
{
    if (in_name != NULL && ext != NULL && out_name != NULL)
    {
        int len = strlen(in_name);
        if (len > 1)
        {
            strcpy(out_name, in_name);
            int dot_pos = fiGetLastPos(in_name, '.');
            if (dot_pos > 0)
                out_name[dot_pos] = 0;
            strcat(out_name, ext);
            return TRUE;
        }
    }
    return FALSE;
}

// renvoie l'extension (avec le '.') correspondant au nom 'full_path'
int fiGetExt(char* full_path, char* ext)
{
    int ret = FALSE;
    if (full_path != NULL && ext != NULL)
    {
        ext[0] = 0;
        int pos = fiGetLastPos(full_path, '.');
        if (pos > 1)
        {
            char* p = &full_path[pos];
            strcpy(ext, p);
            ret = TRUE;
        }
    }
    return ret;
}

// revoie le nom (sans extension) correspondant au nom 'full_path'
int fiGetName(char* full_path, char* name)
{
    int ret = FALSE;
    if (full_path != NULL && name != NULL)
    {
        name[0];
        int pos = fiGetLastPos(full_path, '\\');
        if (pos > 1)
        {
            char* p = &full_path[pos+1];            
            strcpy(name, p);
            pos = fiGetLastPos(name, '.');
            if (pos > 1) name[pos] = 0;
            ret = TRUE;
        }
    }
    return ret;
}

// renvoie le rpertoire correspondant au nom 'full_path'
int fiGetPath(char* full_path, char* path)
{
    int ret = FALSE;
    if (full_path != NULL && path != NULL)
    {
        path[0];
        int pos = fiGetLastPos(full_path, '\\');
        if (pos > 1)
        {
            strncpy(path, full_path, pos);
            ret = TRUE;
        }
    }
    return ret;
}

// renvoie la lettre correspondant 'drive' du fichier 'full_path'
int fiGetDrive(char* full_path, char* drive)
{
    int ret = FALSE;
    if (full_path != NULL && drive != NULL)
    {
        drive[0];
        int pos = fiGetFirstPos(full_path, ':');
        if (pos > 0)
        {
            strncpy(drive, full_path, pos);
            ret = TRUE;
        }
    }
    return ret;
}
//------------------------------------------------------------------------------
//  Exportation des fichiers de donnes
//------------------------------------------------------------------------------
void fi_get_export_extension(int format, char* ext)
// renvoie le nom de la nouvelle extension en fonction du format
{
	if (ext != NULL)
	{
		switch(format)
		{
			case    TRT_FORMAT : strcpy(ext, ".trt"); break;
			case    TXT_FORMAT : strcpy(ext, ".txt"); break;
			case    MAT_FORMAT : strcpy(ext, ".mat"); break;
			case    C3D_FORMAT : strcpy(ext, ".c3d"); break;
		}
	}
}

int fi_export(char* name, char* ev1, char* ev2, int format)
// cette fonction export un fichier de donnes au format dfinie par 'format'
// si ev1 existe (et est non nul), la premire image correspond  l'vnement ev1
// si ev2 existe (et est non nul), la dernire image correspond  l'vnement ev2
// le fichier export contien une matrice pour les donnes cinmatique et 
// une matrice de donnes analogiques (si celles-ci existent)
// chaque matrice contient une premire colonne correspond au temps
// Le fichier export porte le mme nom que le fichier initial, avec une extension'.mat'
{
    if (miGetFrameCount() <=0) return 0;        // test si le fichier est ouvert (donnes existent)
    if (format <0 || format > 3) format = 1;    // controle du format
    char extension[4];
    fi_get_export_extension(format, extension);    // extension du fichier export
    int ret = FALSE;                            // valeur de retour par dfaut
    int first_frame = 0;                        // premire image exporte
    int last_frame = miGetFrameCount() -1;      // dernire image exporte
    int need_to_cut = FALSE;                    // flag pour dcoupage du fichier
    int e1 = -1;                                // frame du premire vnement
    int e2 = -1;                                // frame du deuxime vnement
    if (ev1 != NULL)                            // le nom est non NULL
    {
        e1 = miGetEventFrameFromName(ev1);      // recherche le numro d'image correspondant
        if (e1 >=0)                             // si celle-ci existe
        {
            first_frame = e1;                   // alors on mmorise la premire image
            need_to_cut = TRUE;                 // et on devra 'couper' le fichier
        }
    }
    if (ev2 != NULL)                            // idem pour l'vnement de fin
    {
        e2 = miGetEventFrameFromName(ev2);
        if (e2 > first_frame) 
        {
            last_frame = e2;
            need_to_cut = TRUE;
        }
    }
    if (need_to_cut)                                        // on doit couper le fichier
        miCutBorder(first_frame, last_frame);

    char new_name[MAX_FULLPATH_LEN];                        // nouveau nom du fichier
    if (fiChangeExt(name, extension, new_name))             // on change son extension
    {
        ret = miSaveAs(new_name, format);                   // enregistrement
        if (ret)
            printf("Exportation de %s vers %s\n", name, new_name);
        else
            printf("Erreur de sauvegarde de %s\n", new_name);
    }
    return ret;
}
