//  marker.c
//
//
#include "marker.h"

void mkInitMarker(struct MMARKER* mkr)
{
    if (mkr != NULL)
    {   
        mtInitMatrix(&mkr->mat);        // initialisation de la matrice (vide)
        mkr->name = NULL;               //
        mkr->num = -1;
    }
}

void mkFreeMarker(struct MMARKER* mkr)
{
    if (mkr != NULL)
    {
        mtClearMatrix(&mkr->mat);
        if (mkr->name != NULL) free (mkr->name);
        mkInitMarker(mkr);
    }
}

int mkMarkerIsOk(struct MMARKER* mkr)
{
    int ret = FALSE;
    if (mkr != NULL)
        ret = mtMatrixOk(&(mkr->mat));
    return ret;
}

int mkGetMarkerFromNum(int num, struct MMARKER* mkr)
{
    int ret = FALSE;
    int max_mkr = miGetMarkerCount();
    if (num >=0 && num < max_mkr && mkr != NULL)
    {
        char* name = miGetMarkerName(num);                      // nom du marqueur
        int dim = miGetDimension();                             // nombre de colonnes
        if (dim > 3) dim = 3;
        int len = miGetFrameCount();                            // nombre de lignes

        mkFreeMarker(mkr);                                      // initialisation
        mkr->num = num;                                         // sauve le numr
        mtCreateEmptyMatrix(&mkr->mat, dim);                    // cration des enttes de matrice

        for(int i = 0; i < dim; i++)
            mkr->mat.data[i] = miGetKineDataBuffer(num, i);     //remplissage des donnes
        mkr->mat.row_count = len;                               // mmo du nombre de lignes

        len = strlen(name);                                     // mmo du nom
        mkr->name = malloc(len+1);
        strcpy(mkr->name, name);
        ret = TRUE;
    }
    return ret;
}

int mkGetMarkerFromName(char* name, struct MMARKER* mkr)
{
    int num  = miGetMarkerNum(name);
    return mkGetMarkerFromNum(num, mkr);
}

int mkSaveDataTo(struct MMATRIX* mat, char* name)
{
    int ret = FALSE;
    int num = miGetMarkerNum(name);
    if (mtMatrixOk(mat) && name != NULL && num >=0)
    {
        int len = mat->row_count;
        int dim = mat->col_count;
        if (num >=0 && len > 0 && dim > 0)
        {
            ret = TRUE;
            for(int i = 0; i < dim; i++)
                miPutKineDataBuffer(num, i, mat->data[i], len);
        }
    }
    return ret;
}

int mkSaveMarkerTo(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    if (mkMarkerIsOk(mkr) && name != NULL)
        ret = mkSaveDataTo(&(mkr->mat), name);
    return ret;
}

int mkSaveMarker(struct MMARKER* mkr)
{
    if (mkr != NULL)
        return mkSaveMarkerTo(mkr, mkr->name);
    else printf("Marqueur NULL\n");
    return FALSE;
}

int mkCreateMarker(struct MMARKER* mkr, char* name, char* unit, char* description)
{
    int ret = miNewMarker(name, unit, description); 
    if (ret) 
        ret = mkGetMarkerFromName(name, mkr);
    return ret;
}

int mkCheckNewMarker(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) ret = mkGetMarkerFromNum(num, mkr);
    else ret = mkCreateMarker(mkr, name, "mm", "");
    return ret;
}

int mkCheckNewAngle(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num < 0) ret = mkCreateMarker(mkr, name, "deg", "");
    else ret =  mkGetMarkerFromNum(num, mkr);
    if (ret)
        miNewAngle(name, "deg", "");
    return ret;
}

int mkCheckNewScalar(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) 
        ret = mkGetMarkerFromNum(num, mkr);
    else 
        ret = mkCreateMarker(mkr, name, "", "");
    if (ret)
        miNewScalar(name, "", "");
    return ret;
}

int mkCheckNewForce(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) 
        ret = mkGetMarkerFromNum(num, mkr);
    else 
        ret = mkCreateMarker(mkr, name, "N", "");
    if (ret)
        miNewForce(name, "N", "");
    return ret;
}

int mkCheckNewMoment(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) 
        ret = mkGetMarkerFromNum(num, mkr);
    else 
        ret = mkCreateMarker(mkr, name, "N.mm", "");
    if (ret)
        miNewMoment(name, "N.mm", "");
    return ret;
}

int mkCheckNewPower(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) 
        ret = mkGetMarkerFromNum(num, mkr);
    else 
        ret = mkCreateMarker(mkr, name, "W", "");
    if (ret)
        miNewPower(name, "W", "");
    return ret;
}

int mkCheckNewReaction(struct MMARKER* mkr, char* name)
{
    int ret = FALSE;
    int num  = miGetMarkerNum(name);
    if (num >= 0) 
        ret = mkGetMarkerFromNum(num, mkr);
    else 
        ret = mkCreateMarker(mkr, name, "N", "");
    if (ret)
        miNewReaction(name, "N", "");
    return ret;
}

int mkExists(char* name)
{
    int num  = miGetMarkerNum(name);
    if (num >=0) return TRUE;
    return FALSE;
}

int mkDeleteMarkerFromName(char* name)
{
    if (mkExists(name))
        return miDeleteMarkerFromName(name);
    return FALSE;
}

int mkDeleteMarker(struct MMARKER* mkr)
{
    int ret = FALSE;
    if (mkMarkerIsOk(mkr))
    {
        ret = mkDeleteMarkerFromName(mkr->name);
        mkFreeMarker(mkr);
    }
    return ret;
}