//
//  marker_set.c
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <miAPI.h>

#include "marker_set.h"

int stGetFirstValue(double invalid_value)
// renvoie la premire valeur valide du fichier de donnes
{
    int first = 0;
    int marker_count = miGetMarkerCount();          // nombre total de markers
    int frame_count = 	miGetFrameCount();
    struct MMARKER mkr;
    mkInitMarker(&mkr);
    for(int i = 0; i < marker_count; i++)           // boucle sur les marqueurs
    {
        mkGetMarkerFromNum(i, &mkr);                // recherche le marqueur
        if (mkr.mat.data != NULL)                  // celui-ci est non vide
        {
            if (mkr.name[0] != '*')                 // et le nom ne commence pas par '*' 
            {
                while( mkr.mat.data[0][first] == invalid_value 
                    && mkr.mat.data[1][first] == invalid_value 
                    && mkr.mat.data[2][first] == invalid_value) first++;
            }
        }
    }
    mkFreeMarker(&mkr);                                      
    return first;
}

int stGetLastValue(double invalid_value)
{
    int marker_count = miGetMarkerCount();          // nombre total de markers
    int frame_count = 	miGetFrameCount();
    int last = frame_count-1;
    struct MMARKER mkr;
    mkInitMarker(&mkr);
    for(int i = 0; i < marker_count; i++)           // boucle sur les marqueurs
    {
        mkGetMarkerFromNum(i, &mkr);                // recherche le marqueur
        if (mkr.mat.data != NULL)                  // celui-ci est non vide
        {
            if (mkr.name[0] != '*')                 // et le nom ne commence pas par '*' 
            {
                while(mkr.mat.data[0][last] == invalid_value 
                    && mkr.mat.data[1][last] == invalid_value 
                    && mkr.mat.data[2][last] == invalid_value) last--;
            }
        }
    }
    
    mkFreeMarker(&mkr);                                      
    return last;
}

void stInitSegments(struct MARKER_SET* mkr_set, int count)
// initialisation de la structure des segments avec 'count' segments
{
    if (mkr_set != NULL)
    {
        mkr_set->seg_count = 0;        
        for(int i = 0; i < SEG_MAX_SEG; i++)
        {
            mkr_set->segs[i].count = 0;
            for(int j = 0; j < SEG_MAX_MARKER; j++)
                mkr_set->segs[i].indexes[j] = -1;
        }
        if (count > 0)
            mkr_set->seg_count = count;        
    }
}

void stInitMarkerSet(struct MARKER_SET* mkr_set)
{
    if (mkr_set != NULL)
    {
        mkr_set->names = NULL;
        mkr_set->count = 0;
        mkr_set->seg_count = 0;
        for(int i = 0; i < SEG_MAX_SEG; i++)
        {
            mkr_set->segs[i].count = 0;
            for(int j = 0; j < SEG_MAX_MARKER; j++)
                mkr_set->segs[i].indexes[j] = -1;
            mkr_set->segs[i].name[0]=0;
        }
    }
}


void stClearMarkerSet(struct MARKER_SET* mkr_set)
{
    if (mkr_set != NULL)
    {
        if (mkr_set->names != NULL && mkr_set->count > 0)
        {
            for(int i = 0; i < mkr_set->count; i++)
            {
                free(mkr_set->names[i]);
            }
        }
        stInitMarkerSet(mkr_set);
    }
}

void stPrintMarkerSet(struct MARKER_SET* mkr_set)
{
    if (mkr_set != NULL)
    {
        printf("Marker set\n");
        if (mkr_set->names != NULL && mkr_set->count > 0)
        {
            for(int i = 0; i < mkr_set->count; i++)
            {
                printf("marker[%d] : %s\n", i, mkr_set->names[i]);
            }
        }
        printf("Segments : %d\n", mkr_set->seg_count);
        if (mkr_set->seg_count > 0)
        {
            for(int i = 0; i < mkr_set->seg_count; i++)
            {
                printf("%s : ", mkr_set->segs[i].name);
                for(int j = 0; j < mkr_set->segs[i].count; j++)
                {
                    printf(" %d ", mkr_set->segs[i].indexes[j]);
                }
                printf("\n");
            }
        }
    }
}

void stGetMarkerSetFromData(struct MARKER_SET* mkr_set)
// cette fonction retourne la liste de marqueur  partir du fichier en cours
{
    if (mkr_set != NULL)
    {
        stClearMarkerSet(mkr_set);
        int marker_count = miGetMarkerCount();          // nombre total de markers
        struct MMARKER mkr;
        mkInitMarker(&mkr);
        int local_count = 0;
        int size = 0;
        for(int i = 0; i < marker_count; i++)
        {
            mkGetMarkerFromNum(i, &mkr);                // recherche le marqueur
            if (mkr.mat.data != NULL)                  // celui-ci est non vide
            {
                if (mkr.name[0] != '*')                 // et le nom ne commence pas par '*' 
                {
                    local_count++;
                    int len = strlen(mkr.name);
                    if (len > size) size = len;
                }
            }
        }
        if (local_count > 0 && size > 0)
        {
            mkr_set->count = local_count;
            mkr_set->names = (char**) malloc(local_count*sizeof(char*));
            for(int i = 0; i < local_count; i++)
            {
                mkr_set->names[i] = (char*) malloc(sizeof(char)*size);
            }
            local_count = 0;
            for(int i = 0; i < marker_count; i++)
            {
                mkGetMarkerFromNum(i, &mkr);                // recherche le marqueur
                if (mkr.mat.data != NULL)                  // celui-ci est non vide
                {
                    if (mkr.name[0] != '*' && local_count < mkr_set->count)                 
                    {
                        strcpy(mkr_set->names[local_count], mkr.name);
                        local_count++;
                    }
                }
            }
        }
        mkFreeMarker(&mkr);                                      
    }
}

int stIsInMarkerSet(struct MARKER_SET* mkr_set, char* name)
{
    int ret = -1;
    if (mkr_set != NULL && name != NULL)
    {
        if (mkr_set->names != NULL && mkr_set->count > 0)
        {
            for(int i = 0; i < mkr_set->count; i++)
                if (strcmp(mkr_set->names[i], name) == 0) return i;
        }
    }
    return ret;
}

int stGetLine(char* line, int len, FILE* file)
{
    int count = 0;
    char c = 0;
    line[0]= 0;
    while (!feof(file))
    {
        c = fgetc(file);
        if (c == '\n' || c == '\r') 
        {
            line[count] = 0;
            return count;
        }
        if (count < len)
        {
            line[count++] = c;
            line[count] = 0;
        }
    }
    return count;
}

#define ST_LINE_LEN 1024

int stGetSegmentsFromFile(struct MARKER_SET* mkr_set,char* name)
// lecture des diffrents segment d'un mkr (name)
// un segment est dfinie par une ligne du type : segment_name = makr1,mkr2,mkr3,mkr4
// mkr_set doit dj contenir les diffrents noms des marqueurs
{
    int ret = FALSE;
    if (name == NULL) return false;
    char mkr_name[ST_LINE_LEN];
    
    int p1 = strcspn(name, "\\");
    if (p1 > 1 && p1 < strlen(name))                                // nom avec le rpertoire
    {
        strcpy(mkr_name, name);
    }
    else                                                            // nom simple
    {
        strcpy(mkr_name, moDataDirectory);                          // rpertoire des donnes (variable globale)
        strcat(mkr_name, "\\");
        strcat(mkr_name, name);
    }
    
    FILE * file = fopen(mkr_name, "rt");
    stInitSegments(mkr_set, 0);
    if (file != NULL)                                               // ouverture du fichier
    {
        char buffer[ST_LINE_LEN];
        char tmp[ST_LINE_LEN];
        int ok = TRUE;
        int count_segment = 0;                                      // initialisation du nombre de segments
        while (!feof(file))
        {
            stGetLine(buffer, ST_LINE_LEN, file);                   // lecture d'une ligne du fichier
            strcpy(tmp, buffer);
            
            int count_marker = 0;
            char* p = strtok(buffer, "=");                          // recherche d'une ligne contenant '='
            int n = strcmp(tmp, p);
            if (p!= NULL && n)                                      // nouvelle ligne
            {
                while(p[0] == 32 && strlen(p) > 0) p++;             // remove blank
                while(p[strlen(p)-1] == 32 && strlen(p) > 0) p[strlen(p)-1] = 0;
                if (count_segment < SEG_MAX_SEG )                   // nouveau segment
                {   
                    strcpy(mkr_set->segs[count_segment].name, p);   // nom du segment
                }

                while(p != NULL)                                    // recherche des autres markers
                {
                    p = strtok(NULL, ",");                          // les markers sont spars par des ','
                    if (p != NULL)
                    {
                        while(p[0] == 32 && strlen(p) > 0) p++;
                        while(p[strlen(p)-1] == 32 && strlen(p) > 0) p[strlen(p)-1] = 0;
                        int index = stIsInMarkerSet(mkr_set, p);    // recheche du numro du marqueur
                        if (count_segment < SEG_MAX_SEG && count_marker < SEG_MAX_MARKER && index >= 0)
                        {
                            mkr_set->segs[count_segment].indexes[count_marker] = index;
                            count_marker++;
                        }
                    }
                }
                if (count_marker > 1)                               // si plus d'un marker, c'est un segment
                {
                    if (count_segment < SEG_MAX_SEG)
                        mkr_set->segs[count_segment].count = count_marker;
                    count_segment++;                                // segment suivant
                    if (count_segment <= SEG_MAX_SEG)
                        mkr_set->seg_count = count_segment;
                }
            }
        }
        fclose(file);
        ret = TRUE;
    }
    else
        printf("Impossible d'ouvrir le fichier : %s\n", mkr_name);
    return ret;
}

void stGetMarkerSetFromFile(struct MARKER_SET* mkr_set, char* name)
// lecture du marker set et des segments  partir d'un fichier de type MKR
// le fichier mkr doit avoir le format suivant :
// pour les marker
// NOM_DU_MARKER    Description (le nom est sans espace et la description est spare par une tabulation
//
// pour les segments :
// NOM_DU_SEGMENT = mkr1,mkr2,mkr3  (mkri contient le nom du marqueur, sans espace entre les noms mais avec ','
//

{
    stClearMarkerSet(mkr_set);

    if (name != NULL && mkr_set != NULL)
    {
        char mkr_name[1024];
        int p1 = strcspn(name, "\\");
        if (p1 > 1 && p1 < strlen(name))                                // nom avec le rpertoire
        {
            strcpy(mkr_name, name);
        }
        else                                                            // nom simple
        {
            strcpy(mkr_name, moDataDirectory);                          // rpertoire des donnes (variable globale)
            strcat(mkr_name, "\\");
            strcat(mkr_name, name);
        }

        FILE * file = fopen(mkr_name, "rt");
        char line[1024];
        if (file != NULL)                                           // ouverture du fichier
        {
            int local_count = 0;
            int size = 0;
            int len;
            int end_of_marker = FALSE;
            while (!feof(file))
            {
                stGetLine(line, 1024, file);                        // lecture d'une ligne du fichier
                // lecture des nom des marqueurs : 
                len = strlen(line);
                p1 = strcspn(line, ",");
                if (len > 0 && p1 == len && !end_of_marker)
                {
                    int is_mkr = FALSE;
                    for(int i = 0; i < len-1; i++)                    // enlve les caractre non visibles
                    {
                        if (line[i] <= 32)
                        {
                            if (line[i] == '\t' || line[i] == ' ') is_mkr = TRUE;
                            line[i] = 0;
                            break;
                        }
                    }
                    int mk_id = miGetMarkerNum(line);               // recherche si c'est un nom de marker valide
                    if (mk_id >=0 || is_mkr)
                    {   
                        local_count++;
                        len = strlen(line);
                        if (len > size) size = len;
                    }
                }
                else
                {
                    if (len > 0 && p1 < len) end_of_marker = TRUE;
                }
            }
            if (local_count > 0 && size > 0)
            {
                fseek(file, 0, SEEK_SET);
                mkr_set->count = local_count;
                mkr_set->names = (char**) malloc(local_count*sizeof(char*));
                for(int i = 0; i < local_count; i++)
                {
                    mkr_set->names[i] = (char*) malloc(sizeof(char)*size);
                }
                local_count = 0;
                end_of_marker = FALSE;
                while (!feof(file))
                {
                    stGetLine(line, 1024, file);                        
                    len = strlen(line);
                    p1 = strcspn(line, ",");
                    if (len > 0 && p1 == len && !end_of_marker)
                    {
                        int is_mkr = FALSE;
                        for(int i = 0; i < len; i++)                    // enlve les caractre non visibles
                        {
                            if (line[i] <= 32)
                            {
                                if (line[i] == '\t' || line[i] == ' ') is_mkr = TRUE;
                                line[i] = 0;
                                break;
                            }
                        }
                        int mk_id = miGetMarkerNum(line);               // recherche si c'est un nom de marker valide
                        if ((mk_id >=0  || is_mkr)&& local_count < mkr_set->count)
                        { 
                            strcpy(mkr_set->names[local_count], line);
                            local_count++;
                        }
                    }
                    else
                        if (len > 0 && p1 < len) end_of_marker = TRUE;
                }
            }
            fclose(file);
        }
        stGetSegmentsFromFile(mkr_set, name);
    }
    else
        printf("marker set ou non de fichier vide !\n");
}
