//
//  Implmentation de quelques fonctions Excel utiles
//  
//  Gilles Dietrich, 2014
//

#include <ole_ext.h>            // library OLE 
#include "xls.h"                // prototypes de fonctions

// structure d'une application excel
struct ole_xls {
    struct ole_t* xls;          // object COM
    struct ole_t* xls_app;      // application
    struct ole_t* xls_book;     // feuilles de calcul
};

struct ole_xls excel;           // objet excel

int xlsInit(int show)           // initialisation de l'objet COM
{
    int ret = FALSE;
    excel.xls = ole_new("Excel.Application");
    if (excel.xls != NULL)
    {
        if (show > 0)
            ole_setBoolProperty(excel.xls, "Visible", TRUE);
        excel.xls_app = ole_getObjectProperty(excel.xls, "Application");
        excel.xls_book = ole_getObjectProperty(excel.xls, "WorkBooks");
        if (excel.xls_book != NULL && excel.xls_book != NULL)
        {
            ret = TRUE;
        }
        else printf("Erreur sur l'application excel\n");
    }
    return ret;
}

void xlsQuit()
{
    if (excel.xls != NULL)
    {
        ole_call(excel.xls, "Quit");
        ole_delete(excel.xls);
        excel.xls = NULL;
        excel.xls_app = NULL;
        excel.xls_book = NULL;
    }
}

void xlsAddBook()
{
	if (excel.xls_book != NULL)
        ole_call(excel.xls_book, "Add");
}

void xlsNewBook()
{
	if (excel.xls_book != NULL)
        ole_call(excel.xls_book, "New");
}

void xlsOpen(char* filename)
{
	if (excel.xls_book != NULL && filename != NULL)
        ole_call(excel.xls_book, "Open", filename); 
}

void xlsOpenText(char* filename)
{
	if (excel.xls_book != NULL && filename != NULL)
        ole_call(excel.xls_book, "OpenText", filename); 
}

void xlsClose()
{
	if (excel.xls_book != NULL)
        ole_call(excel.xls_book, "Close"); 
}

void xlsSave()
{
    if (excel.xls_book != NULL)
        ole_call(excel.xls_book, "Save"); 
}

void xlsSaveAs(char* filename)
{
    if (excel.xls_book != NULL)
        ole_call(excel.xls_book, "SaveAs", filename); 
}

int xlsGotoCell(int row, int col)
{
    int ret = FALSE;
    if (excel.xls_app != NULL)
    {
        struct ole_t*  cells = ole_getObjectProperty(excel.xls_app, "Cells", row, col);
        if (cells != NULL)
            ole_call(cells, "Activate");
    }
    return ret;
}

int xlsSetCellString(int row, int col, char* value)
{
    int ret = FALSE;
    if (excel.xls_app != NULL)
    {
        struct ole_t*  cells = ole_getObjectProperty(excel.xls_app, "Cells", row, col);
        if (cells != NULL)
        {
            ole_setStringProperty(cells, "Value", value);
            ret = TRUE;
        }
    }
    return ret;
}

int xlsSetCellInteger(int row, int col, int value)
{
    char s[256];
    sprintf(s, "%d", value);
    return xlsSetCellString(row, col, s);
}

int xlsSetCellDouble(int row, int col, double value)
{
    char s[256];
    sprintf(s, "%g", value);
    return xlsSetCellString(row, col, s);
}

int xlsGetCellValue(int row, int col, double* val)
{
    int ret = FALSE;
    *val = 0;
    if (excel.xls_app != NULL)
    {
        struct ole_t*  cells = ole_getObjectProperty(excel.xls_app, "Cells", row, col);
        if (cells != NULL)
        {
            int type;
            void* value_ptr = ole_getProperty(cells, "Value", &type);
            if (type == OLE_DOUBLE)
            {
                double value = *((double*) value_ptr);
                *val = value;
                ret = TRUE;
            }
            else if(type == OLE_LONG)
            {
                long value = *((long*) value_ptr);
                *val = value;
                ret = TRUE;
            }
        }
    }
    return ret;
}

int xlsGetCellString(int row, int col, char* value, int size)
{
    int ret = FALSE;
    value[0] = 0;
    if (excel.xls_app != NULL)
    {
        struct ole_t*  cells = ole_getObjectProperty(excel.xls_app, "Cells", row, col);
        if (cells != NULL)
        {
            int type;
            void* value_ptr = ole_getProperty(cells, "Value", &type);
            if (type == OLE_STRING)
            {
                char* p = (char*) value_ptr;
                strncpy(value, p, size);
                ret = TRUE;
            }
        }
    }
    return ret;
}
