//  mmatrix.h
//  Quelques fonctions utiles sur les tableaux de donnes (matrice 2D)
//
//  Gilles Dietrich, 2014
#ifndef MMATRIX_H
#define MMATRIX_H

#define		MT_EPS  10e-8			

// vecteur
struct MVECTOR {
    double* data;
    int     row_count;
};

// Matrice
struct MMATRIX {
    double** data;          // pointer sur les donnes
    int     col_count;      // nombre de colonnes
    int     row_count;      // nombre de lignes
};

// initiatlisation d'un vecteur
void    mtInitVector(struct MVECTOR* vec);

// cration d'un vecteur de 'rows' lignes
void    mtCreateVector(struct MVECTOR* vec, int rows);

// libration des donnes du vecteur 'vec'
void    mtClearVector(struct MVECTOR* vec);

int     mtVectorOk(struct MVECTOR* v);
void    mtPrintVector(struct MVECTOR* v);
void    mtZeroVector(struct MVECTOR* v);
double  mtDotProduct(struct MVECTOR* v1, struct MVECTOR* v2);
int     mtCrossProduct(struct MVECTOR* dest, struct MVECTOR* s1, struct MVECTOR* s2);
double  mtNormVector(struct MVECTOR* v);
int     mtNormalizeVector(struct MVECTOR* v);
int     mtCopyVector(struct MVECTOR* dest, struct MVECTOR* source);
int     mtCrossProduct3D(struct MVECTOR* dest, struct MVECTOR* s1, struct MVECTOR* s2);
int     mtScaleVector(struct MVECTOR* v, double s);
int     mtAddVector(struct MVECTOR* dest, struct MVECTOR* s1, struct MVECTOR* s2);
int     mtSubVector(struct MVECTOR* dest, struct MVECTOR* s1, struct MVECTOR* s2);
double  mtDistance(struct MVECTOR* s1, struct MVECTOR* s2);
int     mtMid(struct MVECTOR* dest, struct MVECTOR* s1, struct MVECTOR* s2);

//----------------------------------------------------------------
//  Matrices
//----------------------------------------------------------------
// allocation d'un tableau de donnes (double) ayant 'col' colonnes et 
// 'rows' lignes.
// renvoie le pointeur sur le tableau cr
void    mtCreateMatrix(struct MMATRIX* mat, int cols, int rows);

// allocation des colonnes de la matrice uniquement
void    mtCreateEmptyMatrix(struct MMATRIX* mat, int cols);

// initialisation d'une matrice (plus safe)
void    mtInitMatrix(struct MMATRIX* mat);

// libration d'un tableau de donnes de la matrice 'mat'
void    mtClearMatrix(struct MMATRIX* mat);

// cette fonction ralise la lecture d'un fichier texte (sparateur = Tabulation
// 'filename' contient le nom du fichier  lire, mat correspond  une matrice de donnes
// Renvoie TRUE en cas de succs, sinon FALSE
int     mtReadTabMatrix(char* filename, struct MMATRIX* mat);

// cette fonction crit un tableau de donnes (mat) 
// dans un fichier texte (tabulation)
// Renvoie TRUE en cas de succs, sinon FALSE
int     mtWriteTabMatrix(char* filename, struct MMATRIX* mat);

// lecture d'un fichier texte contenant exclusivement des donnes numriques
// 'filename' est le nom du fichier
// 'mat' est un pointeur sur le tableau de donnes qui sera rempli
// si 'mat' est NULL, la fonction renvoie dans *cols et *rows le nombre de colonnes
// et de lignes du fichier.
// si 'mat' est non NULL, *cols et *rows doivent contenir les dimensions du tableau 'mat' et
// celui-ci contiendra les valeurs lues par cette fonction
// renvoie TRUE en cas de succs, sinon FALSE
int     mtReadTabDataFile(char* filename, double** data, int* cols, int* rows);

// test de la validit d'un matrice. Renvoie TRUE 
int     mtMatrixOk(struct MMATRIX* mat);

// affichage simple de la matrice
void    mtPrintMatrix(struct MMATRIX* mat);

// remise  zro de tous les lments de la matrice
void    mtZeroMatrix(struct MMATRIX* mat);

// matrice identit (si matrice carr
void    mtSetId(struct MMATRIX* mat);

// transpose la matrice 'source' dans 'dest'
int     mtTransposeMatrix(struct MMATRIX* dest, struct MMATRIX* source);

// copie la matrice 'source' dans 'dest'
int     mtCopyMatrix(struct MVECTOR* dest, struct MVECTOR* source);

//  D = s.S
int     mtScaleMatrix(struct MMATRIX* dest, struct MMATRIX* source, double scale);

// D = S1 + S2
int     mtAddMatrix(struct MMATRIX* dest, struct MMATRIX* s1,struct MMATRIX* s2);

// D = S1 - S2
int     mtSubMatrix(struct MMATRIX* dest, struct MMATRIX* s1,struct MMATRIX* s2);

// test si les deux matrices ont la mme taille
int     mtMatrixSameSize(struct MMATRIX* s1,struct MMATRIX* s2);

// dtermine si S est une matrice carr
int     mtIsSquare(struct MMATRIX* s);

// calcul le dterminant d'une matrice 'order' reprsente l'ordre de la matrice
double  mtDeterminant(struct MMATRIX* mat, int order);

// calcul la matrice inverse de 'source'
int     mtInverseSquareMatrix(struct MMATRIX* dest, struct MMATRIX* source);

//----------------------------------------------------------------
//  Matrices et vecteurs
//----------------------------------------------------------------

int     mtMatVecMul(struct MVECTOR* dest, struct MVECTOR* source, struct MMATRIX* mat);

int     mtVectorsToMatrix3D(struct MMATRIX* dest, struct MVECTOR* s1, struct MVECTOR* s2, int a1, int a2);
int     mtFrameToMatrix3D(struct MMATRIX* dest, struct MVECTOR* P1, struct MVECTOR* P2, struct MVECTOR* P3, int a1, int a2);

int     mtToMain3D(struct MVECTOR* dest, struct MVECTOR* source, struct MVECTOR* P0, struct MVECTOR* P1, struct MVECTOR* P2, struct MVECTOR* P3, int a1, int a2);
int     mtToLocal3D(struct MVECTOR* dest, struct MVECTOR* source, struct MVECTOR* P0, struct MVECTOR* P1, struct MVECTOR* P2, struct MVECTOR* P3, int a1, int a2);

#endif
